from __future__ import print_function

import py
from rpython.rlib.parsing.parsing import PackratParser, Rule
from rpython.rlib.parsing.tree import Nonterminal
from rpython.rlib.parsing.regex import StringExpression, RangeExpression
from rpython.rlib.parsing.lexer import Lexer, DummyLexer
from rpython.rlib.parsing.deterministic import compress_char_set, DFA
import string

set = py.builtin.set

ESCAPES = {
    "a": "\a",
    "b": "\b",
    "e": "\x1b",
    "f": "\f",
    "n": "\n",
    "r": "\r",
    "t": "\t",
    "v": "\v",
}

for i in range(256):
    # Add the ctrl-x types:
    #   Rule, according to PCRE:
    #     if x is a lower case letter, it is converted to upper case.
    #     Then bit 6 of the character (hex 40) is inverted.
    #     Thus, \cz => 0x1A, \c{ => 0x3B, \c; => 0x7B.
    escaped = "c%s" % chr(i)
    ESCAPES[escaped] = chr(ord(chr(i).upper()) ^ 0x40)

def unescape_muncher(string):
    """Return a tuple, representing the first character of the string
    (appropriately unescaped) and the rest of the string that wasn't
    handled."""
    if string[0] != '\\':
        # Not an escape character
        return string[0], string[1:]
    if string[1] == 'x':
        # Hex char, must have two hex digits
        char = chr(int(string[2:4], 16))
        return char, string[4:]
    if string[1] in '01234567':
        # Octal number, up to three digits long
        span = 2
        span += (span < len(string)) and (string[span] in '01234567')
        span += (span < len(string)) and (string[span] in '01234567')
        char = chr(int(string[1:span], 8))
        return char, string[span:]
    if string[1] == 'c':
        # Special \cx types
        return ESCAPES['c'+string[2]], string[3:]
    if string[1] in ESCAPES:
        # Special escapes are in ESCAPE
        return ESCAPES[string[1]], string[2:]
    # Otherwise, it's just the character it's meant to be (e.g., '\.')
    return string[1], string[2:]


def unescape(s):
    """Unescape a whole string."""
    result = []
    while s:
        char, s = unescape_muncher(s)
        result.append(char)
    return "".join(result)

syntax =  r"""
EOF:
    !__any__;

parse:
    regex
    [EOF];

regex:
    r1 = concatenation
    '|'
    r2 = regex
    return {r1 | r2}
  | concatenation;

concatenation:
    l = repetition*
    return {reduce(operator.add, l, regex.StringExpression(""))};

repetition:
    r1 = primary
    '*'
    return {r1.kleene()}
  | r1 = primary
    '+'
    return {r1 + r1.kleene()}
  | r1 = primary
    '?'
    return {regex.StringExpression("") | r1}
  | r1 = primary
    '{'
    n = clippednumrange
    '}'
    return {r1 * n + r1.kleene()}
  | r1 = primary
    '{'
    n = numrange
    '}'
    return {r1 * n[0] + reduce(operator.or_, [r1 * i for i in range(n[1] - n[0] + 1)], regex.StringExpression(""))}
  | '{'
    return {regex.StringExpression("{")}
  | primary;

primary:
    ['('] regex [')']
  | range
  | cc = charclass
    return {reduce(operator.or_, [regex.RangeExpression(a, chr(ord(a) + b - 1)) for a, b in compress_char_set(cc)])}
  | c = char
    return {regex.StringExpression(c)}
  | '.'
    return {regex.RangeExpression(chr(0), chr(255))}
  | '-'
    return {regex.StringExpression('-')}
  | '\'
    return {regex.StringExpression('\\')}
  | ']'
    return {regex.StringExpression(']')};

char:
    c = QUOTEDCHAR
    return {unescape(c)}
  | c = CHAR
    return {c};

QUOTEDCHAR:
    `(\\x[0-9a-fA-F]{2})|(\\[0-3]?[0-7][0-7])|(\\c.)|(\\[^dswDSW])`;

CHAR:
    `[^\*\+\(\)\[\]\{\|\.\-\?\^\\]`;

range:
    '['
    s = rangeinner
    ']'
    return {reduce(operator.or_, [regex.RangeExpression(a, chr(ord(a) + b - 1)) for a, b in compress_char_set(s)])};

rangeinner:
    '^'
    s = subrange
    return {set([chr(c) for c in range(256)]) - s}
  | subrange;

subrange:
    ']'
    l = rangeelement*
    return {reduce(operator.or_, [set(["]"])] + l)}
  | l = rangeelement+
    return {reduce(operator.or_, l)};

rangeelement:
    charclass
  | c1 = char
    '-'
    c2 = char
    return {set([chr(i) for i in range(ord(c1), ord(c2) + 1)])}
  | '.'
    return { set(['.']) }
  | '*'
    return { set(['*']) }
  | '+'
    return { set(['+']) }
  | '?'
    return { set(['?']) }
  | '-'
    return { set(['-']) }
  | '['
    return { set(['[']) }
  | c = char
    return { set([c]) };

numrange:
    n1 = NUM
    ','
    n2 = NUM
    return {n1, n2}
  | n1 = NUM
    return {n1, n1};

clippednumrange:
    n1 = NUM
    ','
    return {n1};

charclass:
    '\' 'd'
    return { set([chr(c) for c in range(ord('0'), ord('9')+1)]) }
  | '\' 's'
    return { set(['\t', '\n', '\f', '\r', ' ']) }
  | '\' 'w'
    return { set([chr(c) for c in range(ord('a'), ord('z')+1)] + [chr(c) for c in range(ord('A'), ord('Z')+1)] + [chr(c) for c in range(ord('0'), ord('9')+1)] + ['_']) }
  | '\' 'D'
    return { set([chr(c) for c in range(256)]) - set([chr(c) for c in range(ord('0'), ord('9')+1)]) }
  | '\' 'S'
    return { set([chr(c) for c in range(256)]) - set(['\t', '\n', '\f', '\r', ' ']) }
  | '\' 'W'
    return { set([chr(c) for c in range(256)]) - set([chr(c) for c in range(ord('a'), ord('z')+1)] + [chr(c) for c in range(ord('A'), ord('Z')+1)] + [chr(c) for c in range(ord('0'), ord('9')+1)] + ['_'])};

NUM:
    c = `0|([1-9][0-9]*)`
    return {int(c)};
"""


def parse_regex(s):
    p = RegexParser(s)
    r = p.parse()
    return r

def make_runner(regex, view=False):
    r = parse_regex(regex)
    nfa = r.make_automaton()
    dfa = nfa.make_deterministic()
    if view:
        dfa.view()
    dfa.optimize()
    if view:
        dfa.view()
    r = dfa.get_runner()
    return r














# generated code between this line and its other occurence

from rpython.rlib.parsing.pypackrat import PackratParser, Status
from rpython.rlib.parsing.pypackrat import BacktrackException
from rpython.rlib.parsing import regex
import operator
class Parser(object):
    def EOF(self):
        return self._EOF().result
    def _EOF(self):
        _key = self._pos
        _status = self._dict_EOF.get(_key, None)
        if _status is None:
            _status = self._dict_EOF[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            _choice0 = self._pos
            _stored_result1 = _result
            try:
                _result = self.__any__()
            except BacktrackException:
                self._pos = _choice0
                _result = _stored_result1
            else:
                raise BacktrackException(None)
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._EOF()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = _exc.error
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def parse(self):
        return self._parse().result
    def _parse(self):
        _key = self._pos
        _status = self._dict_parse.get(_key, None)
        if _status is None:
            _status = self._dict_parse[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            _call_status = self._regex()
            _result = _call_status.result
            _error = _call_status.error
            _before_discard0 = _result
            _call_status = self._EOF()
            _result = _call_status.result
            _error = self._combine_errors(_error, _call_status.error)
            _result = _before_discard0
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._parse()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def regex(self):
        return self._regex().result
    def _regex(self):
        _key = self._pos
        _status = self._dict_regex.get(_key, None)
        if _status is None:
            _status = self._dict_regex[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            while 1:
                _choice0 = self._pos
                try:
                    _call_status = self._concatenation()
                    _result = _call_status.result
                    _error = _call_status.error
                    r1 = _result
                    _result = self.__chars__('|')
                    _call_status = self._regex()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    r2 = _result
                    _result = (r1 | r2)
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice0
                _choice1 = self._pos
                try:
                    _call_status = self._concatenation()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice1
                    raise BacktrackException(_error)
                _call_status = self._concatenation()
                _result = _call_status.result
                _error = self._combine_errors(_error, _call_status.error)
                break
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._regex()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def concatenation(self):
        return self._concatenation().result
    def _concatenation(self):
        _key = self._pos
        _status = self._dict_concatenation.get(_key, None)
        if _status is None:
            _status = self._dict_concatenation[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            _all0 = []
            while 1:
                _choice1 = self._pos
                try:
                    _call_status = self._repetition()
                    _result = _call_status.result
                    _error = _call_status.error
                    _all0.append(_result)
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice1
                    break
            _result = _all0
            l = _result
            _result = (reduce(operator.add, l, regex.StringExpression("")))
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._concatenation()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def repetition(self):
        return self._repetition().result
    def _repetition(self):
        _key = self._pos
        _status = self._dict_repetition.get(_key, None)
        if _status is None:
            _status = self._dict_repetition[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            while 1:
                _choice0 = self._pos
                try:
                    _call_status = self._primary()
                    _result = _call_status.result
                    _error = _call_status.error
                    r1 = _result
                    _result = self.__chars__('*')
                    _result = (r1.kleene())
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice0
                _choice1 = self._pos
                try:
                    _call_status = self._primary()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    r1 = _result
                    _result = self.__chars__('+')
                    _result = (r1 + r1.kleene())
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice1
                _choice2 = self._pos
                try:
                    _call_status = self._primary()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    r1 = _result
                    _result = self.__chars__('?')
                    _result = (regex.StringExpression("") | r1)
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice2
                _choice3 = self._pos
                try:
                    _call_status = self._primary()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    r1 = _result
                    _result = self.__chars__('{')
                    _call_status = self._clippednumrange()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    n = _result
                    _result = self.__chars__('}')
                    _result = (r1 * n + r1.kleene())
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice3
                _choice4 = self._pos
                try:
                    _call_status = self._primary()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    r1 = _result
                    _result = self.__chars__('{')
                    _call_status = self._numrange()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    n = _result
                    _result = self.__chars__('}')
                    _result = (r1 * n[0] + reduce(operator.or_, [r1 * i for i in range(n[1] - n[0] + 1)], regex.StringExpression("")))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice4
                _choice5 = self._pos
                try:
                    _result = self.__chars__('{')
                    _result = (regex.StringExpression("{"))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice5
                _choice6 = self._pos
                try:
                    _call_status = self._primary()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice6
                    raise BacktrackException(_error)
                _call_status = self._primary()
                _result = _call_status.result
                _error = self._combine_errors(_error, _call_status.error)
                break
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._repetition()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def primary(self):
        return self._primary().result
    def _primary(self):
        _key = self._pos
        _status = self._dict_primary.get(_key, None)
        if _status is None:
            _status = self._dict_primary[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            while 1:
                _choice0 = self._pos
                try:
                    _before_discard1 = _result
                    _result = self.__chars__('(')
                    _result = _before_discard1
                    _call_status = self._regex()
                    _result = _call_status.result
                    _error = _call_status.error
                    _before_discard2 = _result
                    _result = self.__chars__(')')
                    _result = _before_discard2
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice0
                _choice3 = self._pos
                try:
                    _call_status = self._range()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice3
                _choice4 = self._pos
                try:
                    _call_status = self._charclass()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    cc = _result
                    _result = (reduce(operator.or_, [regex.RangeExpression(a, chr(ord(a) + b - 1)) for a, b in compress_char_set(cc)]))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice4
                _choice5 = self._pos
                try:
                    _call_status = self._char()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    c = _result
                    _result = (regex.StringExpression(c))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice5
                _choice6 = self._pos
                try:
                    _result = self.__chars__('.')
                    _result = (regex.RangeExpression(chr(0), chr(255)))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice6
                _choice7 = self._pos
                try:
                    _result = self.__chars__('-')
                    _result = (regex.StringExpression('-'))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice7
                _choice8 = self._pos
                try:
                    _result = self.__chars__('\\')
                    _result = (regex.StringExpression('\\'))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice8
                _choice9 = self._pos
                try:
                    _result = self.__chars__(']')
                    _result = (regex.StringExpression(']'))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice9
                    raise BacktrackException(_error)
                _result = self.__chars__(']')
                _result = (regex.StringExpression(']'))
                break
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._primary()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def char(self):
        return self._char().result
    def _char(self):
        _key = self._pos
        _status = self._dict_char.get(_key, None)
        if _status is None:
            _status = self._dict_char[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            while 1:
                _choice0 = self._pos
                try:
                    _call_status = self._QUOTEDCHAR()
                    _result = _call_status.result
                    _error = _call_status.error
                    c = _result
                    _result = (unescape(c))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice0
                _choice1 = self._pos
                try:
                    _call_status = self._CHAR()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    c = _result
                    _result = (c)
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice1
                    raise BacktrackException(_error)
                _call_status = self._CHAR()
                _result = _call_status.result
                _error = self._combine_errors(_error, _call_status.error)
                c = _result
                _result = (c)
                break
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._char()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def QUOTEDCHAR(self):
        return self._QUOTEDCHAR().result
    def _QUOTEDCHAR(self):
        _key = self._pos
        _status = self._dict_QUOTEDCHAR.get(_key, None)
        if _status is None:
            _status = self._dict_QUOTEDCHAR[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            _result = self._regex1423754537()
            assert _status.status != _status.LEFTRECURSION
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = _exc.error
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def CHAR(self):
        return self._CHAR().result
    def _CHAR(self):
        _key = self._pos
        _status = self._dict_CHAR.get(_key, None)
        if _status is None:
            _status = self._dict_CHAR[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            _result = self._regex2132196932()
            assert _status.status != _status.LEFTRECURSION
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = _exc.error
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def range(self):
        return self._range().result
    def _range(self):
        _key = self._pos
        _status = self._dict_range.get(_key, None)
        if _status is None:
            _status = self._dict_range[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            _result = self.__chars__('[')
            _call_status = self._rangeinner()
            _result = _call_status.result
            _error = _call_status.error
            s = _result
            _result = self.__chars__(']')
            _result = (reduce(operator.or_, [regex.RangeExpression(a, chr(ord(a) + b - 1)) for a, b in compress_char_set(s)]))
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._range()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def rangeinner(self):
        return self._rangeinner().result
    def _rangeinner(self):
        _key = self._pos
        _status = self._dict_rangeinner.get(_key, None)
        if _status is None:
            _status = self._dict_rangeinner[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            while 1:
                _choice0 = self._pos
                try:
                    _result = self.__chars__('^')
                    _call_status = self._subrange()
                    _result = _call_status.result
                    _error = _call_status.error
                    s = _result
                    _result = (set([chr(c) for c in range(256)]) - s)
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice0
                _choice1 = self._pos
                try:
                    _call_status = self._subrange()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice1
                    raise BacktrackException(_error)
                _call_status = self._subrange()
                _result = _call_status.result
                _error = self._combine_errors(_error, _call_status.error)
                break
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._rangeinner()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def subrange(self):
        return self._subrange().result
    def _subrange(self):
        _key = self._pos
        _status = self._dict_subrange.get(_key, None)
        if _status is None:
            _status = self._dict_subrange[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            while 1:
                _choice0 = self._pos
                try:
                    _result = self.__chars__(']')
                    _all1 = []
                    while 1:
                        _choice2 = self._pos
                        try:
                            _call_status = self._rangeelement()
                            _result = _call_status.result
                            _error = _call_status.error
                            _all1.append(_result)
                        except BacktrackException as _exc:
                            _error = self._combine_errors(_error, _exc.error)
                            self._pos = _choice2
                            break
                    _result = _all1
                    l = _result
                    _result = (reduce(operator.or_, [set(["]"])] + l))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice0
                _choice3 = self._pos
                try:
                    _all4 = []
                    _call_status = self._rangeelement()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    _all4.append(_result)
                    while 1:
                        _choice5 = self._pos
                        try:
                            _call_status = self._rangeelement()
                            _result = _call_status.result
                            _error = self._combine_errors(_error, _call_status.error)
                            _all4.append(_result)
                        except BacktrackException as _exc:
                            _error = self._combine_errors(_error, _exc.error)
                            self._pos = _choice5
                            break
                    _result = _all4
                    l = _result
                    _result = (reduce(operator.or_, l))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice3
                    raise BacktrackException(_error)
                _all6 = []
                _call_status = self._rangeelement()
                _result = _call_status.result
                _error = self._combine_errors(_error, _call_status.error)
                _all6.append(_result)
                while 1:
                    _choice7 = self._pos
                    try:
                        _call_status = self._rangeelement()
                        _result = _call_status.result
                        _error = self._combine_errors(_error, _call_status.error)
                        _all6.append(_result)
                    except BacktrackException as _exc:
                        _error = self._combine_errors(_error, _exc.error)
                        self._pos = _choice7
                        break
                _result = _all6
                l = _result
                _result = (reduce(operator.or_, l))
                break
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._subrange()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def rangeelement(self):
        return self._rangeelement().result
    def _rangeelement(self):
        _key = self._pos
        _status = self._dict_rangeelement.get(_key, None)
        if _status is None:
            _status = self._dict_rangeelement[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            while 1:
                _choice0 = self._pos
                try:
                    _call_status = self._charclass()
                    _result = _call_status.result
                    _error = _call_status.error
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice0
                _choice1 = self._pos
                try:
                    _call_status = self._char()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    c1 = _result
                    _result = self.__chars__('-')
                    _call_status = self._char()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    c2 = _result
                    _result = (set([chr(i) for i in range(ord(c1), ord(c2) + 1)]))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice1
                _choice2 = self._pos
                try:
                    _result = self.__chars__('.')
                    _result = ( set(['.']) )
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice2
                _choice3 = self._pos
                try:
                    _result = self.__chars__('*')
                    _result = ( set(['*']) )
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice3
                _choice4 = self._pos
                try:
                    _result = self.__chars__('+')
                    _result = ( set(['+']) )
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice4
                _choice5 = self._pos
                try:
                    _result = self.__chars__('?')
                    _result = ( set(['?']) )
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice5
                _choice6 = self._pos
                try:
                    _result = self.__chars__('-')
                    _result = ( set(['-']) )
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice6
                _choice7 = self._pos
                try:
                    _result = self.__chars__('[')
                    _result = ( set(['[']) )
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice7
                _choice8 = self._pos
                try:
                    _call_status = self._char()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    c = _result
                    _result = ( set([c]) )
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice8
                    raise BacktrackException(_error)
                _call_status = self._char()
                _result = _call_status.result
                _error = self._combine_errors(_error, _call_status.error)
                c = _result
                _result = ( set([c]) )
                break
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._rangeelement()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def numrange(self):
        return self._numrange().result
    def _numrange(self):
        _key = self._pos
        _status = self._dict_numrange.get(_key, None)
        if _status is None:
            _status = self._dict_numrange[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            while 1:
                _choice0 = self._pos
                try:
                    _call_status = self._NUM()
                    _result = _call_status.result
                    _error = _call_status.error
                    n1 = _result
                    _result = self.__chars__(',')
                    _call_status = self._NUM()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    n2 = _result
                    _result = (n1, n2)
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice0
                _choice1 = self._pos
                try:
                    _call_status = self._NUM()
                    _result = _call_status.result
                    _error = self._combine_errors(_error, _call_status.error)
                    n1 = _result
                    _result = (n1, n1)
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice1
                    raise BacktrackException(_error)
                _call_status = self._NUM()
                _result = _call_status.result
                _error = self._combine_errors(_error, _call_status.error)
                n1 = _result
                _result = (n1, n1)
                break
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._numrange()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def clippednumrange(self):
        return self._clippednumrange().result
    def _clippednumrange(self):
        _key = self._pos
        _status = self._dict_clippednumrange.get(_key, None)
        if _status is None:
            _status = self._dict_clippednumrange[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
            elif (_statusstatus == _status.INPROGRESS or
                  _statusstatus == _status.LEFTRECURSION):
                _status.status = _status.LEFTRECURSION
                if _status.result is not None:
                    self._pos = _status.pos
                    return _status
                else:
                    raise BacktrackException(None)
            elif _statusstatus == _status.SOMESOLUTIONS:
                _status.status = _status.INPROGRESS
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            _call_status = self._NUM()
            _result = _call_status.result
            _error = _call_status.error
            n1 = _result
            _result = self.__chars__(',')
            _result = (n1)
            if _status.status == _status.LEFTRECURSION:
                if _status.result is not None:
                    if _status.pos >= self._pos:
                        _status.status = _status.NORMAL
                        self._pos = _status.pos
                        return _status
                _status.pos = self._pos
                _status.status = _status.SOMESOLUTIONS
                _status.result = _result
                _status.error = _error
                self._pos = _startingpos
                return self._clippednumrange()
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def charclass(self):
        return self._charclass().result
    def _charclass(self):
        _key = self._pos
        _status = self._dict_charclass.get(_key, None)
        if _status is None:
            _status = self._dict_charclass[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            while 1:
                _choice0 = self._pos
                try:
                    _result = self.__chars__('\\')
                    _result = self.__chars__('d')
                    _result = ( set([chr(c) for c in range(ord('0'), ord('9')+1)]) )
                    break
                except BacktrackException as _exc:
                    _error = _exc.error
                    self._pos = _choice0
                _choice1 = self._pos
                try:
                    _result = self.__chars__('\\')
                    _result = self.__chars__('s')
                    _result = ( set(['\t', '\n', '\f', '\r', ' ']) )
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice1
                _choice2 = self._pos
                try:
                    _result = self.__chars__('\\')
                    _result = self.__chars__('w')
                    _result = ( set([chr(c) for c in range(ord('a'), ord('z')+1)] + [chr(c) for c in range(ord('A'), ord('Z')+1)] + [chr(c) for c in range(ord('0'), ord('9')+1)] + ['_']) )
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice2
                _choice3 = self._pos
                try:
                    _result = self.__chars__('\\')
                    _result = self.__chars__('D')
                    _result = ( set([chr(c) for c in range(256)]) - set([chr(c) for c in range(ord('0'), ord('9')+1)]) )
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice3
                _choice4 = self._pos
                try:
                    _result = self.__chars__('\\')
                    _result = self.__chars__('S')
                    _result = ( set([chr(c) for c in range(256)]) - set(['\t', '\n', '\f', '\r', ' ']) )
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice4
                _choice5 = self._pos
                try:
                    _result = self.__chars__('\\')
                    _result = self.__chars__('W')
                    _result = ( set([chr(c) for c in range(256)]) - set([chr(c) for c in range(ord('a'), ord('z')+1)] + [chr(c) for c in range(ord('A'), ord('Z')+1)] + [chr(c) for c in range(ord('0'), ord('9')+1)] + ['_']))
                    break
                except BacktrackException as _exc:
                    _error = self._combine_errors(_error, _exc.error)
                    self._pos = _choice5
                    raise BacktrackException(_error)
                _result = self.__chars__('\\')
                _result = self.__chars__('W')
                _result = ( set([chr(c) for c in range(256)]) - set([chr(c) for c in range(ord('a'), ord('z')+1)] + [chr(c) for c in range(ord('A'), ord('Z')+1)] + [chr(c) for c in range(ord('0'), ord('9')+1)] + ['_']))
                break
            assert _status.status != _status.LEFTRECURSION
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = self._combine_errors(_error, _exc.error)
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def NUM(self):
        return self._NUM().result
    def _NUM(self):
        _key = self._pos
        _status = self._dict_NUM.get(_key, None)
        if _status is None:
            _status = self._dict_NUM[_key] = Status()
        else:
            _statusstatus = _status.status
            if _statusstatus == _status.NORMAL:
                self._pos = _status.pos
                return _status
            elif _statusstatus == _status.ERROR:
                raise BacktrackException(_status.error)
        _startingpos = self._pos
        try:
            _result = None
            _error = None
            _result = self._regex1166214427()
            c = _result
            _result = (int(c))
            assert _status.status != _status.LEFTRECURSION
            _status.status = _status.NORMAL
            _status.pos = self._pos
            _status.result = _result
            _status.error = _error
            return _status
        except BacktrackException as _exc:
            _status.pos = -1
            _status.result = None
            _error = _exc.error
            _status.error = _error
            _status.status = _status.ERROR
            raise BacktrackException(_error)
    def __init__(self, inputstream):
        self._dict_EOF = {}
        self._dict_parse = {}
        self._dict_regex = {}
        self._dict_concatenation = {}
        self._dict_repetition = {}
        self._dict_primary = {}
        self._dict_char = {}
        self._dict_QUOTEDCHAR = {}
        self._dict_CHAR = {}
        self._dict_range = {}
        self._dict_rangeinner = {}
        self._dict_subrange = {}
        self._dict_rangeelement = {}
        self._dict_numrange = {}
        self._dict_clippednumrange = {}
        self._dict_charclass = {}
        self._dict_NUM = {}
        self._pos = 0
        self._inputstream = inputstream
    def _regex2132196932(self):
        _choice0 = self._pos
        _runner = self._Runner(self._inputstream, self._pos)
        _i = _runner.recognize_2132196932(self._pos)
        if _runner.last_matched_state == -1:
            self._pos = _choice0
            raise BacktrackException
        _upto = _runner.last_matched_index + 1
        _pos = self._pos
        assert _pos >= 0
        assert _upto >= 0
        _result = self._inputstream[_pos: _upto]
        self._pos = _upto
        return _result
    def _regex1166214427(self):
        _choice1 = self._pos
        _runner = self._Runner(self._inputstream, self._pos)
        _i = _runner.recognize_1166214427(self._pos)
        if _runner.last_matched_state == -1:
            self._pos = _choice1
            raise BacktrackException
        _upto = _runner.last_matched_index + 1
        _pos = self._pos
        assert _pos >= 0
        assert _upto >= 0
        _result = self._inputstream[_pos: _upto]
        self._pos = _upto
        return _result
    def _regex1423754537(self):
        _choice2 = self._pos
        _runner = self._Runner(self._inputstream, self._pos)
        _i = _runner.recognize_1423754537(self._pos)
        if _runner.last_matched_state == -1:
            self._pos = _choice2
            raise BacktrackException
        _upto = _runner.last_matched_index + 1
        _pos = self._pos
        assert _pos >= 0
        assert _upto >= 0
        _result = self._inputstream[_pos: _upto]
        self._pos = _upto
        return _result
    class _Runner(object):
        def __init__(self, text, pos):
            self.text = text
            self.pos = pos
            self.last_matched_state = -1
            self.last_matched_index = -1
            self.state = -1
        def recognize_2132196932(runner, i):
            #auto-generated code, don't edit
            assert i >= 0
            input = runner.text
            state = 0
            while 1:
                if state == 0:
                    try:
                        char = input[i]
                        i += 1
                    except IndexError:
                        runner.state = 0
                        return ~i
                    if '}' <= char <= '\xff':
                        state = 1
                    elif '\x00' <= char <= "'":
                        state = 1
                    elif '_' <= char <= 'z':
                        state = 1
                    elif '@' <= char <= 'Z':
                        state = 1
                    elif '/' <= char <= '>':
                        state = 1
                    elif char == ',':
                        state = 1
                    else:
                        break
                runner.last_matched_state = state
                runner.last_matched_index = i - 1
                runner.state = state
                if i == len(input):
                    return i
                else:
                    return ~i
                break
            runner.state = state
            return ~i
        def recognize_1166214427(runner, i):
            #auto-generated code, don't edit
            assert i >= 0
            input = runner.text
            state = 0
            while 1:
                if state == 0:
                    try:
                        char = input[i]
                        i += 1
                    except IndexError:
                        runner.state = 0
                        return ~i
                    if char == '0':
                        state = 1
                    elif '1' <= char <= '9':
                        state = 2
                    else:
                        break
                if state == 2:
                    runner.last_matched_index = i - 1
                    runner.last_matched_state = state
                    try:
                        char = input[i]
                        i += 1
                    except IndexError:
                        runner.state = 2
                        return i
                    if '0' <= char <= '9':
                        state = 2
                        continue
                    else:
                        break
                runner.last_matched_state = state
                runner.last_matched_index = i - 1
                runner.state = state
                if i == len(input):
                    return i
                else:
                    return ~i
                break
            runner.state = state
            return ~i
        def recognize_1423754537(runner, i):
            #auto-generated code, don't edit
            assert i >= 0
            input = runner.text
            state = 0
            while 1:
                if state == 0:
                    try:
                        char = input[i]
                        i += 1
                    except IndexError:
                        runner.state = 0
                        return ~i
                    if char == '\\':
                        state = 5
                    else:
                        break
                if state == 1:
                    runner.last_matched_index = i - 1
                    runner.last_matched_state = state
                    try:
                        char = input[i]
                        i += 1
                    except IndexError:
                        runner.state = 1
                        return i
                    if '0' <= char <= '9':
                        state = 6
                    elif 'A' <= char <= 'F':
                        state = 6
                    elif 'a' <= char <= 'f':
                        state = 6
                    else:
                        break
                if state == 2:
                    runner.last_matched_index = i - 1
                    runner.last_matched_state = state
                    try:
                        char = input[i]
                        i += 1
                    except IndexError:
                        runner.state = 2
                        return i
                    if '0' <= char <= '7':
                        state = 4
                    else:
                        break
                if state == 3:
                    runner.last_matched_index = i - 1
                    runner.last_matched_state = state
                    try:
                        char = input[i]
                        i += 1
                    except IndexError:
                        runner.state = 3
                        return i
                    if '0' <= char <= '7':
                        state = 2
                        continue
                    else:
                        break
                if state == 5:
                    try:
                        char = input[i]
                        i += 1
                    except IndexError:
                        runner.state = 5
                        return ~i
                    if char == 'x':
                        state = 1
                        continue
                    elif '4' <= char <= '7':
                        state = 2
                        continue
                    elif '0' <= char <= '3':
                        state = 3
                        continue
                    elif 'y' <= char <= '\xff':
                        state = 4
                    elif '\x00' <= char <= '/':
                        state = 4
                    elif 'E' <= char <= 'R':
                        state = 4
                    elif 'e' <= char <= 'r':
                        state = 4
                    elif '8' <= char <= 'C':
                        state = 4
                    elif 'X' <= char <= 'b':
                        state = 4
                    elif 'T' <= char <= 'V':
                        state = 4
                    elif 't' <= char <= 'v':
                        state = 4
                    elif char == 'c':
                        state = 7
                    else:
                        break
                if state == 6:
                    try:
                        char = input[i]
                        i += 1
                    except IndexError:
                        runner.state = 6
                        return ~i
                    if '0' <= char <= '9':
                        state = 4
                    elif 'A' <= char <= 'F':
                        state = 4
                    elif 'a' <= char <= 'f':
                        state = 4
                    else:
                        break
                if state == 7:
                    runner.last_matched_index = i - 1
                    runner.last_matched_state = state
                    try:
                        char = input[i]
                        i += 1
                    except IndexError:
                        runner.state = 7
                        return i
                    if '\x00' <= char <= '\xff':
                        state = 4
                    else:
                        break
                runner.last_matched_state = state
                runner.last_matched_index = i - 1
                runner.state = state
                if i == len(input):
                    return i
                else:
                    return ~i
                break
            runner.state = state
            return ~i
class RegexParser(PackratParser):
    def __init__(self, stream):
        self.init_parser(stream)
forbidden = dict.fromkeys(("__weakref__ __doc__ "
                           "__dict__ __module__").split())
initthere = "__init__" in RegexParser.__dict__
for key, value in Parser.__dict__.iteritems():
    if key not in RegexParser.__dict__ and key not in forbidden:
        setattr(RegexParser, key, value)
RegexParser.init_parser = Parser.__init__.im_func
# generated code between this line and its other occurence























































def test_generate():
    f = py.path.local(__file__)
    oldcontent = f.read()
    s = "# GENERATED CODE BETWEEN THIS LINE AND ITS OTHER OCCURENCE\n".lower()
    pre, gen, after = oldcontent.split(s)

    from pypackrat import PyPackratSyntaxParser
    from makepackrat import TreeOptimizer, ParserBuilder
    p = PyPackratSyntaxParser(syntax)
    t = p.file()
    t = t.visit(TreeOptimizer())
    visitor = ParserBuilder()
    t.visit(visitor)
    code = visitor.get_code()
    content = """\
%s\
%s
from rpython.rlib.parsing.pypackrat import PackratParser, Status
from rpython.rlib.parsing.pypackrat import BacktrackException
from rpython.rlib.parsing import regex
import operator
%s
class RegexParser(PackratParser):
    def __init__(self, stream):
        self.init_parser(stream)
forbidden = dict.fromkeys(("__weakref__ __doc__ "
                           "__dict__ __module__").split())
initthere = "__init__" in RegexParser.__dict__
for key, value in Parser.__dict__.iteritems():
    if key not in RegexParser.__dict__ and key not in forbidden:
        setattr(RegexParser, key, value)
RegexParser.init_parser = Parser.__init__.im_func
%s
%s\
""" % (pre, s, code, s, after)
    print(content)
    f.write(content)



# Generated by ruleopt/generate.py, don't edit!

import sys
from rpython.jit.metainterp.history import ConstInt
from rpython.jit.metainterp.optimizeopt.util import (
    get_box_replacement)
from rpython.jit.metainterp.resoperation import rop

from rpython.rlib.rarithmetic import LONG_BIT, r_uint, intmask, ovfcheck, uint_mul_high, highest_bit
MAXINT = sys.maxint
MININT = -sys.maxint - 1


class OptIntAutoGenerated(object):
    def _eq(self, box1, bound1, box2, bound2):
        if box1 is box2: return True
        if bound1.is_constant() and bound2.is_constant() and bound1.lower == bound2.lower: return True
        return False
    _all_rules_fired = []
    _rule_names_int_add = ['add_reassoc_consts', 'add_zero', 'add_sub_x_c_c', 'add_sub_c_x_c']
    _rule_fired_int_add = [0] * 4
    _all_rules_fired.append(('int_add', _rule_names_int_add, _rule_fired_int_add))
    def optimize_INT_ADD(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_1 = get_box_replacement(op.getarg(1))
        b_arg_1 = self.getintbound(arg_1)
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # add_zero: int_add(0, x) => x
            if C_arg_0 == 0:
                self.make_equal_to(op, arg_1)
                self._rule_fired_int_add[1] += 1
                return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # add_zero: int_add(x, 0) => x
            if C_arg_1 == 0:
                self.make_equal_to(op, arg_0)
                self._rule_fired_int_add[1] += 1
                return
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            arg_1_int_add = self.optimizer.as_operation(arg_1, rop.INT_ADD)
            if arg_1_int_add is not None:
                arg_1_0 = get_box_replacement(arg_1_int_add.getarg(0))
                b_arg_1_0 = self.getintbound(arg_1_0)
                arg_1_1 = get_box_replacement(arg_1_int_add.getarg(1))
                b_arg_1_1 = self.getintbound(arg_1_1)
                if b_arg_1_0.is_constant():
                    C_arg_1_0 = b_arg_1_0.get_constant_int()
                    # add_reassoc_consts: int_add(C2, int_add(C1, x)) => int_add(x, C)
                    C = intmask(r_uint(C_arg_1_0) + r_uint(C_arg_0))
                    newop = self.replace_op_with(op, rop.INT_ADD, args=[arg_1_1, ConstInt(C)])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_add[0] += 1
                    return
                if b_arg_1_1.is_constant():
                    C_arg_1_1 = b_arg_1_1.get_constant_int()
                    # add_reassoc_consts: int_add(C2, int_add(x, C1)) => int_add(x, C)
                    C = intmask(r_uint(C_arg_1_1) + r_uint(C_arg_0))
                    newop = self.replace_op_with(op, rop.INT_ADD, args=[arg_1_0, ConstInt(C)])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_add[0] += 1
                    return
            else:
                arg_1_int_sub = self.optimizer.as_operation(arg_1, rop.INT_SUB)
                if arg_1_int_sub is not None:
                    arg_1_0 = get_box_replacement(arg_1_int_sub.getarg(0))
                    b_arg_1_0 = self.getintbound(arg_1_0)
                    arg_1_1 = get_box_replacement(arg_1_int_sub.getarg(1))
                    b_arg_1_1 = self.getintbound(arg_1_1)
                    if b_arg_1_0.is_constant():
                        C_arg_1_0 = b_arg_1_0.get_constant_int()
                        # add_sub_c_x_c: int_add(C2, int_sub(C1, x)) => int_sub(C, x)
                        C = intmask(r_uint(C_arg_1_0) + r_uint(C_arg_0))
                        newop = self.replace_op_with(op, rop.INT_SUB, args=[ConstInt(C), arg_1_1])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_add[3] += 1
                        return
                    if b_arg_1_1.is_constant():
                        C_arg_1_1 = b_arg_1_1.get_constant_int()
                        # add_sub_x_c_c: int_add(C2, int_sub(x, C1)) => int_add(x, C)
                        C = intmask(r_uint(C_arg_0) - r_uint(C_arg_1_1))
                        newop = self.replace_op_with(op, rop.INT_ADD, args=[arg_1_0, ConstInt(C)])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_add[2] += 1
                        return
        else:
            arg_0_int_add = self.optimizer.as_operation(arg_0, rop.INT_ADD)
            if arg_0_int_add is not None:
                arg_0_0 = get_box_replacement(arg_0_int_add.getarg(0))
                b_arg_0_0 = self.getintbound(arg_0_0)
                arg_0_1 = get_box_replacement(arg_0_int_add.getarg(1))
                b_arg_0_1 = self.getintbound(arg_0_1)
                if b_arg_0_0.is_constant():
                    C_arg_0_0 = b_arg_0_0.get_constant_int()
                    if b_arg_1.is_constant():
                        C_arg_1 = b_arg_1.get_constant_int()
                        # add_reassoc_consts: int_add(int_add(C1, x), C2) => int_add(x, C)
                        C = intmask(r_uint(C_arg_0_0) + r_uint(C_arg_1))
                        newop = self.replace_op_with(op, rop.INT_ADD, args=[arg_0_1, ConstInt(C)])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_add[0] += 1
                        return
                if b_arg_0_1.is_constant():
                    C_arg_0_1 = b_arg_0_1.get_constant_int()
                    if b_arg_1.is_constant():
                        C_arg_1 = b_arg_1.get_constant_int()
                        # add_reassoc_consts: int_add(int_add(x, C1), C2) => int_add(x, C)
                        C = intmask(r_uint(C_arg_0_1) + r_uint(C_arg_1))
                        newop = self.replace_op_with(op, rop.INT_ADD, args=[arg_0_0, ConstInt(C)])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_add[0] += 1
                        return
            else:
                arg_0_int_sub = self.optimizer.as_operation(arg_0, rop.INT_SUB)
                if arg_0_int_sub is not None:
                    arg_0_0 = get_box_replacement(arg_0_int_sub.getarg(0))
                    b_arg_0_0 = self.getintbound(arg_0_0)
                    arg_0_1 = get_box_replacement(arg_0_int_sub.getarg(1))
                    b_arg_0_1 = self.getintbound(arg_0_1)
                    if b_arg_0_0.is_constant():
                        C_arg_0_0 = b_arg_0_0.get_constant_int()
                        if b_arg_1.is_constant():
                            C_arg_1 = b_arg_1.get_constant_int()
                            # add_sub_c_x_c: int_add(int_sub(C1, x), C2) => int_sub(C, x)
                            C = intmask(r_uint(C_arg_0_0) + r_uint(C_arg_1))
                            newop = self.replace_op_with(op, rop.INT_SUB, args=[ConstInt(C), arg_0_1])
                            self.optimizer.send_extra_operation(newop)
                            self._rule_fired_int_add[3] += 1
                            return
                    if b_arg_0_1.is_constant():
                        C_arg_0_1 = b_arg_0_1.get_constant_int()
                        if b_arg_1.is_constant():
                            C_arg_1 = b_arg_1.get_constant_int()
                            # add_sub_x_c_c: int_add(int_sub(x, C1), C2) => int_add(x, C)
                            C = intmask(r_uint(C_arg_1) - r_uint(C_arg_0_1))
                            newop = self.replace_op_with(op, rop.INT_ADD, args=[arg_0_0, ConstInt(C)])
                            self.optimizer.send_extra_operation(newop)
                            self._rule_fired_int_add[2] += 1
                            return
        return self.emit(op)
    _rule_names_int_sub = ['sub_zero', 'sub_from_zero', 'sub_x_x', 'sub_add_consts', 'sub_add', 'sub_add_neg', 'sub_sub_left_x_c_c', 'sub_sub_left_c_x_c', 'sub_xor_x_y_y', 'sub_or_x_y_y', 'sub_invert_one']
    _rule_fired_int_sub = [0] * 11
    _all_rules_fired.append(('int_sub', _rule_names_int_sub, _rule_fired_int_sub))
    def optimize_INT_SUB(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_1 = get_box_replacement(op.getarg(1))
        b_arg_1 = self.getintbound(arg_1)
        # sub_x_x: int_sub(x, x) => 0
        if self._eq(arg_1, b_arg_1, arg_0, b_arg_0):
            self.make_constant_int(op, 0)
            self._rule_fired_int_sub[2] += 1
            return
        arg_0_int_add = self.optimizer.as_operation(arg_0, rop.INT_ADD)
        if arg_0_int_add is not None:
            arg_0_0 = get_box_replacement(arg_0_int_add.getarg(0))
            b_arg_0_0 = self.getintbound(arg_0_0)
            arg_0_1 = get_box_replacement(arg_0_int_add.getarg(1))
            b_arg_0_1 = self.getintbound(arg_0_1)
            # sub_add: int_sub(int_add(x, y), y) => x
            if self._eq(arg_1, b_arg_1, arg_0_1, b_arg_0_1):
                self.make_equal_to(op, arg_0_0)
                self._rule_fired_int_sub[4] += 1
                return
            # sub_add: int_sub(int_add(y, x), y) => x
            if self._eq(arg_1, b_arg_1, arg_0_0, b_arg_0_0):
                self.make_equal_to(op, arg_0_1)
                self._rule_fired_int_sub[4] += 1
                return
        else:
            arg_0_int_or = self.optimizer.as_operation(arg_0, rop.INT_OR)
            if arg_0_int_or is not None:
                arg_0_0 = get_box_replacement(arg_0_int_or.getarg(0))
                b_arg_0_0 = self.getintbound(arg_0_0)
                arg_0_1 = get_box_replacement(arg_0_int_or.getarg(1))
                b_arg_0_1 = self.getintbound(arg_0_1)
                # sub_or_x_y_y: int_sub(int_or(x, y), y) => x
                if self._eq(arg_1, b_arg_1, arg_0_1, b_arg_0_1):
                    if b_arg_0_0.and_bound(b_arg_0_1).known_eq_const(0):
                        self.make_equal_to(op, arg_0_0)
                        self._rule_fired_int_sub[9] += 1
                        return
                # sub_or_x_y_y: int_sub(int_or(y, x), y) => x
                if self._eq(arg_1, b_arg_1, arg_0_0, b_arg_0_0):
                    if b_arg_0_1.and_bound(b_arg_0_0).known_eq_const(0):
                        self.make_equal_to(op, arg_0_1)
                        self._rule_fired_int_sub[9] += 1
                        return
            else:
                arg_0_int_xor = self.optimizer.as_operation(arg_0, rop.INT_XOR)
                if arg_0_int_xor is not None:
                    arg_0_0 = get_box_replacement(arg_0_int_xor.getarg(0))
                    b_arg_0_0 = self.getintbound(arg_0_0)
                    arg_0_1 = get_box_replacement(arg_0_int_xor.getarg(1))
                    b_arg_0_1 = self.getintbound(arg_0_1)
                    # sub_xor_x_y_y: int_sub(int_xor(x, y), y) => x
                    if self._eq(arg_1, b_arg_1, arg_0_1, b_arg_0_1):
                        if b_arg_0_0.and_bound(b_arg_0_1).known_eq_const(0):
                            self.make_equal_to(op, arg_0_0)
                            self._rule_fired_int_sub[8] += 1
                            return
                    # sub_xor_x_y_y: int_sub(int_xor(y, x), y) => x
                    if self._eq(arg_1, b_arg_1, arg_0_0, b_arg_0_0):
                        if b_arg_0_1.and_bound(b_arg_0_0).known_eq_const(0):
                            self.make_equal_to(op, arg_0_1)
                            self._rule_fired_int_sub[8] += 1
                            return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # sub_zero: int_sub(x, 0) => x
            if C_arg_1 == 0:
                self.make_equal_to(op, arg_0)
                self._rule_fired_int_sub[0] += 1
                return
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # sub_from_zero: int_sub(0, x) => int_neg(x)
            if C_arg_0 == 0:
                newop = self.replace_op_with(op, rop.INT_NEG, args=[arg_1])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_sub[1] += 1
                return
        else:
            arg_0_int_invert = self.optimizer.as_operation(arg_0, rop.INT_INVERT)
            if arg_0_int_invert is not None:
                arg_0_0 = get_box_replacement(arg_0_int_invert.getarg(0))
                b_arg_0_0 = self.getintbound(arg_0_0)
                if b_arg_1.is_constant():
                    C_arg_1 = b_arg_1.get_constant_int()
                    # sub_invert_one: int_sub(int_invert(x), -1) => int_neg(x)
                    if C_arg_1 == -1:
                        newop = self.replace_op_with(op, rop.INT_NEG, args=[arg_0_0])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_sub[10] += 1
                        return
            else:
                arg_0_int_add = self.optimizer.as_operation(arg_0, rop.INT_ADD)
                if arg_0_int_add is not None:
                    arg_0_0 = get_box_replacement(arg_0_int_add.getarg(0))
                    b_arg_0_0 = self.getintbound(arg_0_0)
                    arg_0_1 = get_box_replacement(arg_0_int_add.getarg(1))
                    b_arg_0_1 = self.getintbound(arg_0_1)
                    if b_arg_0_0.is_constant():
                        C_arg_0_0 = b_arg_0_0.get_constant_int()
                        if b_arg_1.is_constant():
                            C_arg_1 = b_arg_1.get_constant_int()
                            # sub_add_consts: int_sub(int_add(C1, x), C2) => int_sub(x, C)
                            C = intmask(r_uint(C_arg_1) - r_uint(C_arg_0_0))
                            newop = self.replace_op_with(op, rop.INT_SUB, args=[arg_0_1, ConstInt(C)])
                            self.optimizer.send_extra_operation(newop)
                            self._rule_fired_int_sub[3] += 1
                            return
                    if b_arg_0_1.is_constant():
                        C_arg_0_1 = b_arg_0_1.get_constant_int()
                        if b_arg_1.is_constant():
                            C_arg_1 = b_arg_1.get_constant_int()
                            # sub_add_consts: int_sub(int_add(x, C1), C2) => int_sub(x, C)
                            C = intmask(r_uint(C_arg_1) - r_uint(C_arg_0_1))
                            newop = self.replace_op_with(op, rop.INT_SUB, args=[arg_0_0, ConstInt(C)])
                            self.optimizer.send_extra_operation(newop)
                            self._rule_fired_int_sub[3] += 1
                            return
                else:
                    arg_0_int_sub = self.optimizer.as_operation(arg_0, rop.INT_SUB)
                    if arg_0_int_sub is not None:
                        arg_0_0 = get_box_replacement(arg_0_int_sub.getarg(0))
                        b_arg_0_0 = self.getintbound(arg_0_0)
                        arg_0_1 = get_box_replacement(arg_0_int_sub.getarg(1))
                        b_arg_0_1 = self.getintbound(arg_0_1)
                        if b_arg_0_0.is_constant():
                            C_arg_0_0 = b_arg_0_0.get_constant_int()
                            if b_arg_1.is_constant():
                                C_arg_1 = b_arg_1.get_constant_int()
                                # sub_sub_left_c_x_c: int_sub(int_sub(C1, x), C2) => int_sub(C, x)
                                C = intmask(r_uint(C_arg_0_0) - r_uint(C_arg_1))
                                newop = self.replace_op_with(op, rop.INT_SUB, args=[ConstInt(C), arg_0_1])
                                self.optimizer.send_extra_operation(newop)
                                self._rule_fired_int_sub[7] += 1
                                return
                        if b_arg_0_1.is_constant():
                            C_arg_0_1 = b_arg_0_1.get_constant_int()
                            if b_arg_1.is_constant():
                                C_arg_1 = b_arg_1.get_constant_int()
                                # sub_sub_left_x_c_c: int_sub(int_sub(x, C1), C2) => int_sub(x, C)
                                C = intmask(r_uint(C_arg_0_1) + r_uint(C_arg_1))
                                newop = self.replace_op_with(op, rop.INT_SUB, args=[arg_0_0, ConstInt(C)])
                                self.optimizer.send_extra_operation(newop)
                                self._rule_fired_int_sub[6] += 1
                                return
        arg_1_int_add = self.optimizer.as_operation(arg_1, rop.INT_ADD)
        if arg_1_int_add is not None:
            arg_1_0 = get_box_replacement(arg_1_int_add.getarg(0))
            b_arg_1_0 = self.getintbound(arg_1_0)
            arg_1_1 = get_box_replacement(arg_1_int_add.getarg(1))
            b_arg_1_1 = self.getintbound(arg_1_1)
            # sub_add_neg: int_sub(y, int_add(x, y)) => int_neg(x)
            if self._eq(arg_1_1, b_arg_1_1, arg_0, b_arg_0):
                newop = self.replace_op_with(op, rop.INT_NEG, args=[arg_1_0])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_sub[5] += 1
                return
            # sub_add_neg: int_sub(y, int_add(y, x)) => int_neg(x)
            if self._eq(arg_1_0, b_arg_1_0, arg_0, b_arg_0):
                newop = self.replace_op_with(op, rop.INT_NEG, args=[arg_1_1])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_sub[5] += 1
                return
        return self.emit(op)
    _rule_names_int_mul = ['mul_zero', 'mul_one', 'mul_minus_one', 'mul_pow2_const', 'mul_lshift']
    _rule_fired_int_mul = [0] * 5
    _all_rules_fired.append(('int_mul', _rule_names_int_mul, _rule_fired_int_mul))
    def optimize_INT_MUL(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_1 = get_box_replacement(op.getarg(1))
        b_arg_1 = self.getintbound(arg_1)
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # mul_zero: int_mul(0, x) => 0
            if C_arg_0 == 0:
                self.make_constant_int(op, 0)
                self._rule_fired_int_mul[0] += 1
                return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # mul_zero: int_mul(x, 0) => 0
            if C_arg_1 == 0:
                self.make_constant_int(op, 0)
                self._rule_fired_int_mul[0] += 1
                return
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # mul_one: int_mul(1, x) => x
            if C_arg_0 == 1:
                self.make_equal_to(op, arg_1)
                self._rule_fired_int_mul[1] += 1
                return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # mul_one: int_mul(x, 1) => x
            if C_arg_1 == 1:
                self.make_equal_to(op, arg_0)
                self._rule_fired_int_mul[1] += 1
                return
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # mul_minus_one: int_mul(-1, x) => int_neg(x)
            if C_arg_0 == -1:
                newop = self.replace_op_with(op, rop.INT_NEG, args=[arg_1])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_mul[2] += 1
                return
            # mul_pow2_const: int_mul(C, x) => int_lshift(x, shift)
            if C_arg_0 > 0 and C_arg_0 & intmask(r_uint(C_arg_0) - r_uint(1)) == 0:
                shift = highest_bit(C_arg_0)
                newop = self.replace_op_with(op, rop.INT_LSHIFT, args=[arg_1, ConstInt(shift)])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_mul[3] += 1
                return
        else:
            arg_0_int_lshift = self.optimizer.as_operation(arg_0, rop.INT_LSHIFT)
            if arg_0_int_lshift is not None:
                arg_0_0 = get_box_replacement(arg_0_int_lshift.getarg(0))
                b_arg_0_0 = self.getintbound(arg_0_0)
                arg_0_1 = get_box_replacement(arg_0_int_lshift.getarg(1))
                b_arg_0_1 = self.getintbound(arg_0_1)
                if b_arg_0_0.is_constant():
                    C_arg_0_0 = b_arg_0_0.get_constant_int()
                    # mul_lshift: int_mul(int_lshift(1, y), x) => int_lshift(x, y)
                    if C_arg_0_0 == 1:
                        if b_arg_0_1.known_ge_const(0) and b_arg_0_1.known_le_const(LONG_BIT):
                            newop = self.replace_op_with(op, rop.INT_LSHIFT, args=[arg_1, arg_0_1])
                            self.optimizer.send_extra_operation(newop)
                            self._rule_fired_int_mul[4] += 1
                            return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # mul_minus_one: int_mul(x, -1) => int_neg(x)
            if C_arg_1 == -1:
                newop = self.replace_op_with(op, rop.INT_NEG, args=[arg_0])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_mul[2] += 1
                return
            # mul_pow2_const: int_mul(x, C) => int_lshift(x, shift)
            if C_arg_1 > 0 and C_arg_1 & intmask(r_uint(C_arg_1) - r_uint(1)) == 0:
                shift = highest_bit(C_arg_1)
                newop = self.replace_op_with(op, rop.INT_LSHIFT, args=[arg_0, ConstInt(shift)])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_mul[3] += 1
                return
        else:
            arg_1_int_lshift = self.optimizer.as_operation(arg_1, rop.INT_LSHIFT)
            if arg_1_int_lshift is not None:
                arg_1_0 = get_box_replacement(arg_1_int_lshift.getarg(0))
                b_arg_1_0 = self.getintbound(arg_1_0)
                arg_1_1 = get_box_replacement(arg_1_int_lshift.getarg(1))
                b_arg_1_1 = self.getintbound(arg_1_1)
                if b_arg_1_0.is_constant():
                    C_arg_1_0 = b_arg_1_0.get_constant_int()
                    # mul_lshift: int_mul(x, int_lshift(1, y)) => int_lshift(x, y)
                    if C_arg_1_0 == 1:
                        if b_arg_1_1.known_ge_const(0) and b_arg_1_1.known_le_const(LONG_BIT):
                            newop = self.replace_op_with(op, rop.INT_LSHIFT, args=[arg_0, arg_1_1])
                            self.optimizer.send_extra_operation(newop)
                            self._rule_fired_int_mul[4] += 1
                            return
        return self.emit(op)
    _rule_names_int_and = ['and_x_x', 'and_reassoc_consts', 'and_x_c_in_range', 'and_idempotent', 'and_or', 'and_known_result', 'and_absorb']
    _rule_fired_int_and = [0] * 7
    _all_rules_fired.append(('int_and', _rule_names_int_and, _rule_fired_int_and))
    def optimize_INT_AND(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_1 = get_box_replacement(op.getarg(1))
        b_arg_1 = self.getintbound(arg_1)
        # and_known_result: int_and(a, b) => C
        bound = b_arg_0.and_bound(b_arg_1)
        if bound.is_constant():
            C = bound.get_constant_int()
            self.make_equal_to(op, ConstInt(C))
            self._rule_fired_int_and[5] += 1
            return
        # and_known_result: int_and(b, a) => C
        bound = b_arg_1.and_bound(b_arg_0)
        if bound.is_constant():
            C = bound.get_constant_int()
            self.make_equal_to(op, ConstInt(C))
            self._rule_fired_int_and[5] += 1
            return
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # and_x_c_in_range: int_and(C, x) => x
            if intmask(b_arg_1.lower) >= 0 and intmask(b_arg_1.upper) <= C_arg_0 & ~intmask(r_uint(C_arg_0) + r_uint(1)):
                self.make_equal_to(op, arg_1)
                self._rule_fired_int_and[2] += 1
                return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # and_x_c_in_range: int_and(x, C) => x
            if intmask(b_arg_0.lower) >= 0 and intmask(b_arg_0.upper) <= C_arg_1 & ~intmask(r_uint(C_arg_1) + r_uint(1)):
                self.make_equal_to(op, arg_0)
                self._rule_fired_int_and[2] += 1
                return
        # and_x_x: int_and(a, a) => a
        if self._eq(arg_1, b_arg_1, arg_0, b_arg_0):
            self.make_equal_to(op, arg_0)
            self._rule_fired_int_and[0] += 1
            return
        # and_idempotent: int_and(x, y) => x
        if intmask(b_arg_1.tvalue) | intmask(~(b_arg_0.tvalue | b_arg_0.tmask)) == -1:
            self.make_equal_to(op, arg_0)
            self._rule_fired_int_and[3] += 1
            return
        # and_idempotent: int_and(y, x) => x
        if intmask(b_arg_0.tvalue) | intmask(~(b_arg_1.tvalue | b_arg_1.tmask)) == -1:
            self.make_equal_to(op, arg_1)
            self._rule_fired_int_and[3] += 1
            return
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            arg_1_int_and = self.optimizer.as_operation(arg_1, rop.INT_AND)
            if arg_1_int_and is not None:
                arg_1_0 = get_box_replacement(arg_1_int_and.getarg(0))
                b_arg_1_0 = self.getintbound(arg_1_0)
                arg_1_1 = get_box_replacement(arg_1_int_and.getarg(1))
                b_arg_1_1 = self.getintbound(arg_1_1)
                if b_arg_1_0.is_constant():
                    C_arg_1_0 = b_arg_1_0.get_constant_int()
                    # and_reassoc_consts: int_and(C2, int_and(C1, x)) => int_and(x, C)
                    C = C_arg_1_0 & C_arg_0
                    newop = self.replace_op_with(op, rop.INT_AND, args=[arg_1_1, ConstInt(C)])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_and[1] += 1
                    return
                if b_arg_1_1.is_constant():
                    C_arg_1_1 = b_arg_1_1.get_constant_int()
                    # and_reassoc_consts: int_and(C2, int_and(x, C1)) => int_and(x, C)
                    C = C_arg_1_1 & C_arg_0
                    newop = self.replace_op_with(op, rop.INT_AND, args=[arg_1_0, ConstInt(C)])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_and[1] += 1
                    return
        else:
            arg_0_int_and = self.optimizer.as_operation(arg_0, rop.INT_AND)
            if arg_0_int_and is not None:
                arg_0_0 = get_box_replacement(arg_0_int_and.getarg(0))
                b_arg_0_0 = self.getintbound(arg_0_0)
                arg_0_1 = get_box_replacement(arg_0_int_and.getarg(1))
                b_arg_0_1 = self.getintbound(arg_0_1)
                if b_arg_0_0.is_constant():
                    C_arg_0_0 = b_arg_0_0.get_constant_int()
                    if b_arg_1.is_constant():
                        C_arg_1 = b_arg_1.get_constant_int()
                        # and_reassoc_consts: int_and(int_and(C1, x), C2) => int_and(x, C)
                        C = C_arg_0_0 & C_arg_1
                        newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_1, ConstInt(C)])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_and[1] += 1
                        return
                if b_arg_0_1.is_constant():
                    C_arg_0_1 = b_arg_0_1.get_constant_int()
                    if b_arg_1.is_constant():
                        C_arg_1 = b_arg_1.get_constant_int()
                        # and_reassoc_consts: int_and(int_and(x, C1), C2) => int_and(x, C)
                        C = C_arg_0_1 & C_arg_1
                        newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_0, ConstInt(C)])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_and[1] += 1
                        return
                # and_absorb: int_and(int_and(a, b), a) => int_and(a, b)
                if self._eq(arg_1, b_arg_1, arg_0_0, b_arg_0_0):
                    newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_0, arg_0_1])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_and[6] += 1
                    return
                # and_absorb: int_and(int_and(b, a), a) => int_and(a, b)
                if self._eq(arg_1, b_arg_1, arg_0_1, b_arg_0_1):
                    newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_1, arg_0_0])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_and[6] += 1
                    return
            else:
                arg_0_int_or = self.optimizer.as_operation(arg_0, rop.INT_OR)
                if arg_0_int_or is not None:
                    arg_0_0 = get_box_replacement(arg_0_int_or.getarg(0))
                    b_arg_0_0 = self.getintbound(arg_0_0)
                    arg_0_1 = get_box_replacement(arg_0_int_or.getarg(1))
                    b_arg_0_1 = self.getintbound(arg_0_1)
                    # and_or: int_and(int_or(x, y), z) => int_and(x, z)
                    if b_arg_0_1.and_bound(b_arg_1).known_eq_const(0):
                        newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_0, arg_1])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_and[4] += 1
                        return
                    # and_or: int_and(int_or(y, x), z) => int_and(x, z)
                    if b_arg_0_0.and_bound(b_arg_1).known_eq_const(0):
                        newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_1, arg_1])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_and[4] += 1
                        return
        arg_1_int_and = self.optimizer.as_operation(arg_1, rop.INT_AND)
        if arg_1_int_and is not None:
            arg_1_0 = get_box_replacement(arg_1_int_and.getarg(0))
            b_arg_1_0 = self.getintbound(arg_1_0)
            arg_1_1 = get_box_replacement(arg_1_int_and.getarg(1))
            b_arg_1_1 = self.getintbound(arg_1_1)
            # and_absorb: int_and(a, int_and(a, b)) => int_and(a, b)
            if self._eq(arg_1_0, b_arg_1_0, arg_0, b_arg_0):
                newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0, arg_1_1])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_and[6] += 1
                return
            # and_absorb: int_and(a, int_and(b, a)) => int_and(a, b)
            if self._eq(arg_1_1, b_arg_1_1, arg_0, b_arg_0):
                newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0, arg_1_0])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_and[6] += 1
                return
        else:
            arg_1_int_or = self.optimizer.as_operation(arg_1, rop.INT_OR)
            if arg_1_int_or is not None:
                arg_1_0 = get_box_replacement(arg_1_int_or.getarg(0))
                b_arg_1_0 = self.getintbound(arg_1_0)
                arg_1_1 = get_box_replacement(arg_1_int_or.getarg(1))
                b_arg_1_1 = self.getintbound(arg_1_1)
                # and_or: int_and(z, int_or(x, y)) => int_and(x, z)
                if b_arg_1_1.and_bound(b_arg_0).known_eq_const(0):
                    newop = self.replace_op_with(op, rop.INT_AND, args=[arg_1_0, arg_0])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_and[4] += 1
                    return
                # and_or: int_and(z, int_or(y, x)) => int_and(x, z)
                if b_arg_1_0.and_bound(b_arg_0).known_eq_const(0):
                    newop = self.replace_op_with(op, rop.INT_AND, args=[arg_1_1, arg_0])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_and[4] += 1
                    return
        return self.emit(op)
    _rule_names_int_or = ['or_x_x', 'or_known_result', 'or_absorb', 'or_idempotent', 'or_and_two_parts', 'or_reassoc_consts']
    _rule_fired_int_or = [0] * 6
    _all_rules_fired.append(('int_or', _rule_names_int_or, _rule_fired_int_or))
    def optimize_INT_OR(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_1 = get_box_replacement(op.getarg(1))
        b_arg_1 = self.getintbound(arg_1)
        # or_known_result: int_or(a, b) => C
        bound = b_arg_0.or_bound(b_arg_1)
        if bound.is_constant():
            C = bound.get_constant_int()
            self.make_equal_to(op, ConstInt(C))
            self._rule_fired_int_or[1] += 1
            return
        # or_known_result: int_or(b, a) => C
        bound = b_arg_1.or_bound(b_arg_0)
        if bound.is_constant():
            C = bound.get_constant_int()
            self.make_equal_to(op, ConstInt(C))
            self._rule_fired_int_or[1] += 1
            return
        # or_x_x: int_or(a, a) => a
        if self._eq(arg_1, b_arg_1, arg_0, b_arg_0):
            self.make_equal_to(op, arg_0)
            self._rule_fired_int_or[0] += 1
            return
        # or_idempotent: int_or(x, y) => x
        if intmask(b_arg_0.tvalue) | intmask(~(b_arg_1.tvalue | b_arg_1.tmask)) == -1:
            self.make_equal_to(op, arg_0)
            self._rule_fired_int_or[3] += 1
            return
        # or_idempotent: int_or(y, x) => x
        if intmask(b_arg_1.tvalue) | intmask(~(b_arg_0.tvalue | b_arg_0.tmask)) == -1:
            self.make_equal_to(op, arg_1)
            self._rule_fired_int_or[3] += 1
            return
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            arg_1_int_or = self.optimizer.as_operation(arg_1, rop.INT_OR)
            if arg_1_int_or is not None:
                arg_1_0 = get_box_replacement(arg_1_int_or.getarg(0))
                b_arg_1_0 = self.getintbound(arg_1_0)
                arg_1_1 = get_box_replacement(arg_1_int_or.getarg(1))
                b_arg_1_1 = self.getintbound(arg_1_1)
                if b_arg_1_0.is_constant():
                    C_arg_1_0 = b_arg_1_0.get_constant_int()
                    # or_reassoc_consts: int_or(C2, int_or(C1, x)) => int_or(x, C)
                    C = C_arg_1_0 | C_arg_0
                    newop = self.replace_op_with(op, rop.INT_OR, args=[arg_1_1, ConstInt(C)])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_or[5] += 1
                    return
                if b_arg_1_1.is_constant():
                    C_arg_1_1 = b_arg_1_1.get_constant_int()
                    # or_reassoc_consts: int_or(C2, int_or(x, C1)) => int_or(x, C)
                    C = C_arg_1_1 | C_arg_0
                    newop = self.replace_op_with(op, rop.INT_OR, args=[arg_1_0, ConstInt(C)])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_or[5] += 1
                    return
        else:
            arg_0_int_and = self.optimizer.as_operation(arg_0, rop.INT_AND)
            if arg_0_int_and is not None:
                arg_0_0 = get_box_replacement(arg_0_int_and.getarg(0))
                b_arg_0_0 = self.getintbound(arg_0_0)
                arg_0_1 = get_box_replacement(arg_0_int_and.getarg(1))
                b_arg_0_1 = self.getintbound(arg_0_1)
                if b_arg_0_0.is_constant():
                    C_arg_0_0 = b_arg_0_0.get_constant_int()
                    arg_1_int_and = self.optimizer.as_operation(arg_1, rop.INT_AND)
                    if arg_1_int_and is not None:
                        arg_1_0 = get_box_replacement(arg_1_int_and.getarg(0))
                        b_arg_1_0 = self.getintbound(arg_1_0)
                        arg_1_1 = get_box_replacement(arg_1_int_and.getarg(1))
                        b_arg_1_1 = self.getintbound(arg_1_1)
                        if b_arg_1_0.is_constant():
                            C_arg_1_0 = b_arg_1_0.get_constant_int()
                            # or_and_two_parts: int_or(int_and(C1, x), int_and(C2, x)) => int_and(x, C)
                            if self._eq(arg_1_1, b_arg_1_1, arg_0_1, b_arg_0_1):
                                C = C_arg_0_0 | C_arg_1_0
                                newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_1, ConstInt(C)])
                                self.optimizer.send_extra_operation(newop)
                                self._rule_fired_int_or[4] += 1
                                return
                            # or_and_two_parts: int_or(int_and(C2, x), int_and(C1, x)) => int_and(x, C)
                            if self._eq(arg_1_1, b_arg_1_1, arg_0_1, b_arg_0_1):
                                C = C_arg_1_0 | C_arg_0_0
                                newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_1, ConstInt(C)])
                                self.optimizer.send_extra_operation(newop)
                                self._rule_fired_int_or[4] += 1
                                return
                        if b_arg_1_1.is_constant():
                            C_arg_1_1 = b_arg_1_1.get_constant_int()
                            # or_and_two_parts: int_or(int_and(C2, x), int_and(x, C1)) => int_and(x, C)
                            if self._eq(arg_1_0, b_arg_1_0, arg_0_1, b_arg_0_1):
                                C = C_arg_1_1 | C_arg_0_0
                                newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_1, ConstInt(C)])
                                self.optimizer.send_extra_operation(newop)
                                self._rule_fired_int_or[4] += 1
                                return
                            # or_and_two_parts: int_or(int_and(C1, x), int_and(x, C2)) => int_and(x, C)
                            if self._eq(arg_1_0, b_arg_1_0, arg_0_1, b_arg_0_1):
                                C = C_arg_0_0 | C_arg_1_1
                                newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_1, ConstInt(C)])
                                self.optimizer.send_extra_operation(newop)
                                self._rule_fired_int_or[4] += 1
                                return
                if b_arg_0_1.is_constant():
                    C_arg_0_1 = b_arg_0_1.get_constant_int()
                    arg_1_int_and = self.optimizer.as_operation(arg_1, rop.INT_AND)
                    if arg_1_int_and is not None:
                        arg_1_0 = get_box_replacement(arg_1_int_and.getarg(0))
                        b_arg_1_0 = self.getintbound(arg_1_0)
                        arg_1_1 = get_box_replacement(arg_1_int_and.getarg(1))
                        b_arg_1_1 = self.getintbound(arg_1_1)
                        if b_arg_1_0.is_constant():
                            C_arg_1_0 = b_arg_1_0.get_constant_int()
                            # or_and_two_parts: int_or(int_and(x, C1), int_and(C2, x)) => int_and(x, C)
                            if self._eq(arg_1_1, b_arg_1_1, arg_0_0, b_arg_0_0):
                                C = C_arg_0_1 | C_arg_1_0
                                newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_0, ConstInt(C)])
                                self.optimizer.send_extra_operation(newop)
                                self._rule_fired_int_or[4] += 1
                                return
                            # or_and_two_parts: int_or(int_and(x, C2), int_and(C1, x)) => int_and(x, C)
                            if self._eq(arg_1_1, b_arg_1_1, arg_0_0, b_arg_0_0):
                                C = C_arg_1_0 | C_arg_0_1
                                newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_0, ConstInt(C)])
                                self.optimizer.send_extra_operation(newop)
                                self._rule_fired_int_or[4] += 1
                                return
                        if b_arg_1_1.is_constant():
                            C_arg_1_1 = b_arg_1_1.get_constant_int()
                            # or_and_two_parts: int_or(int_and(x, C1), int_and(x, C2)) => int_and(x, C)
                            if self._eq(arg_1_0, b_arg_1_0, arg_0_0, b_arg_0_0):
                                C = C_arg_0_1 | C_arg_1_1
                                newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_0, ConstInt(C)])
                                self.optimizer.send_extra_operation(newop)
                                self._rule_fired_int_or[4] += 1
                                return
                            # or_and_two_parts: int_or(int_and(x, C2), int_and(x, C1)) => int_and(x, C)
                            if self._eq(arg_1_0, b_arg_1_0, arg_0_0, b_arg_0_0):
                                C = C_arg_1_1 | C_arg_0_1
                                newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_0, ConstInt(C)])
                                self.optimizer.send_extra_operation(newop)
                                self._rule_fired_int_or[4] += 1
                                return
            else:
                arg_0_int_or = self.optimizer.as_operation(arg_0, rop.INT_OR)
                if arg_0_int_or is not None:
                    arg_0_0 = get_box_replacement(arg_0_int_or.getarg(0))
                    b_arg_0_0 = self.getintbound(arg_0_0)
                    arg_0_1 = get_box_replacement(arg_0_int_or.getarg(1))
                    b_arg_0_1 = self.getintbound(arg_0_1)
                    if b_arg_0_0.is_constant():
                        C_arg_0_0 = b_arg_0_0.get_constant_int()
                        if b_arg_1.is_constant():
                            C_arg_1 = b_arg_1.get_constant_int()
                            # or_reassoc_consts: int_or(int_or(C1, x), C2) => int_or(x, C)
                            C = C_arg_0_0 | C_arg_1
                            newop = self.replace_op_with(op, rop.INT_OR, args=[arg_0_1, ConstInt(C)])
                            self.optimizer.send_extra_operation(newop)
                            self._rule_fired_int_or[5] += 1
                            return
                    if b_arg_0_1.is_constant():
                        C_arg_0_1 = b_arg_0_1.get_constant_int()
                        if b_arg_1.is_constant():
                            C_arg_1 = b_arg_1.get_constant_int()
                            # or_reassoc_consts: int_or(int_or(x, C1), C2) => int_or(x, C)
                            C = C_arg_0_1 | C_arg_1
                            newop = self.replace_op_with(op, rop.INT_OR, args=[arg_0_0, ConstInt(C)])
                            self.optimizer.send_extra_operation(newop)
                            self._rule_fired_int_or[5] += 1
                            return
                    # or_absorb: int_or(int_or(a, b), a) => int_or(a, b)
                    if self._eq(arg_1, b_arg_1, arg_0_0, b_arg_0_0):
                        newop = self.replace_op_with(op, rop.INT_OR, args=[arg_0_0, arg_0_1])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_or[2] += 1
                        return
                    # or_absorb: int_or(int_or(b, a), a) => int_or(a, b)
                    if self._eq(arg_1, b_arg_1, arg_0_1, b_arg_0_1):
                        newop = self.replace_op_with(op, rop.INT_OR, args=[arg_0_1, arg_0_0])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_or[2] += 1
                        return
        arg_1_int_or = self.optimizer.as_operation(arg_1, rop.INT_OR)
        if arg_1_int_or is not None:
            arg_1_0 = get_box_replacement(arg_1_int_or.getarg(0))
            b_arg_1_0 = self.getintbound(arg_1_0)
            arg_1_1 = get_box_replacement(arg_1_int_or.getarg(1))
            b_arg_1_1 = self.getintbound(arg_1_1)
            # or_absorb: int_or(a, int_or(a, b)) => int_or(a, b)
            if self._eq(arg_1_0, b_arg_1_0, arg_0, b_arg_0):
                newop = self.replace_op_with(op, rop.INT_OR, args=[arg_0, arg_1_1])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_or[2] += 1
                return
            # or_absorb: int_or(a, int_or(b, a)) => int_or(a, b)
            if self._eq(arg_1_1, b_arg_1_1, arg_0, b_arg_0):
                newop = self.replace_op_with(op, rop.INT_OR, args=[arg_0, arg_1_0])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_or[2] += 1
                return
        return self.emit(op)
    _rule_names_int_xor = ['xor_x_x', 'xor_reassoc_consts', 'xor_absorb', 'xor_zero', 'xor_minus_1', 'xor_known_result', 'xor_is_not']
    _rule_fired_int_xor = [0] * 7
    _all_rules_fired.append(('int_xor', _rule_names_int_xor, _rule_fired_int_xor))
    def optimize_INT_XOR(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_1 = get_box_replacement(op.getarg(1))
        b_arg_1 = self.getintbound(arg_1)
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            if b_arg_1.is_constant():
                C_arg_1 = b_arg_1.get_constant_int()
                # xor_known_result: int_xor(C1, C2) => C
                C = C_arg_0 ^ C_arg_1
                self.make_equal_to(op, ConstInt(C))
                self._rule_fired_int_xor[5] += 1
                return
                # xor_known_result: int_xor(C2, C1) => C
                C = C_arg_1 ^ C_arg_0
                self.make_equal_to(op, ConstInt(C))
                self._rule_fired_int_xor[5] += 1
                return
        # xor_x_x: int_xor(a, a) => 0
        if self._eq(arg_1, b_arg_1, arg_0, b_arg_0):
            self.make_constant_int(op, 0)
            self._rule_fired_int_xor[0] += 1
            return
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # xor_zero: int_xor(0, a) => a
            if C_arg_0 == 0:
                self.make_equal_to(op, arg_1)
                self._rule_fired_int_xor[3] += 1
                return
        else:
            arg_0_int_xor = self.optimizer.as_operation(arg_0, rop.INT_XOR)
            if arg_0_int_xor is not None:
                arg_0_0 = get_box_replacement(arg_0_int_xor.getarg(0))
                b_arg_0_0 = self.getintbound(arg_0_0)
                arg_0_1 = get_box_replacement(arg_0_int_xor.getarg(1))
                b_arg_0_1 = self.getintbound(arg_0_1)
                # xor_absorb: int_xor(int_xor(a, b), b) => a
                if self._eq(arg_1, b_arg_1, arg_0_1, b_arg_0_1):
                    self.make_equal_to(op, arg_0_0)
                    self._rule_fired_int_xor[2] += 1
                    return
                # xor_absorb: int_xor(int_xor(b, a), b) => a
                if self._eq(arg_1, b_arg_1, arg_0_0, b_arg_0_0):
                    self.make_equal_to(op, arg_0_1)
                    self._rule_fired_int_xor[2] += 1
                    return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # xor_zero: int_xor(a, 0) => a
            if C_arg_1 == 0:
                self.make_equal_to(op, arg_0)
                self._rule_fired_int_xor[3] += 1
                return
        else:
            arg_1_int_xor = self.optimizer.as_operation(arg_1, rop.INT_XOR)
            if arg_1_int_xor is not None:
                arg_1_0 = get_box_replacement(arg_1_int_xor.getarg(0))
                b_arg_1_0 = self.getintbound(arg_1_0)
                arg_1_1 = get_box_replacement(arg_1_int_xor.getarg(1))
                b_arg_1_1 = self.getintbound(arg_1_1)
                # xor_absorb: int_xor(b, int_xor(a, b)) => a
                if self._eq(arg_1_1, b_arg_1_1, arg_0, b_arg_0):
                    self.make_equal_to(op, arg_1_0)
                    self._rule_fired_int_xor[2] += 1
                    return
                # xor_absorb: int_xor(b, int_xor(b, a)) => a
                if self._eq(arg_1_0, b_arg_1_0, arg_0, b_arg_0):
                    self.make_equal_to(op, arg_1_1)
                    self._rule_fired_int_xor[2] += 1
                    return
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            arg_1_int_xor = self.optimizer.as_operation(arg_1, rop.INT_XOR)
            if arg_1_int_xor is not None:
                arg_1_0 = get_box_replacement(arg_1_int_xor.getarg(0))
                b_arg_1_0 = self.getintbound(arg_1_0)
                arg_1_1 = get_box_replacement(arg_1_int_xor.getarg(1))
                b_arg_1_1 = self.getintbound(arg_1_1)
                if b_arg_1_0.is_constant():
                    C_arg_1_0 = b_arg_1_0.get_constant_int()
                    # xor_reassoc_consts: int_xor(C2, int_xor(C1, x)) => int_xor(x, C)
                    C = C_arg_1_0 ^ C_arg_0
                    newop = self.replace_op_with(op, rop.INT_XOR, args=[arg_1_1, ConstInt(C)])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_xor[1] += 1
                    return
                if b_arg_1_1.is_constant():
                    C_arg_1_1 = b_arg_1_1.get_constant_int()
                    # xor_reassoc_consts: int_xor(C2, int_xor(x, C1)) => int_xor(x, C)
                    C = C_arg_1_1 ^ C_arg_0
                    newop = self.replace_op_with(op, rop.INT_XOR, args=[arg_1_0, ConstInt(C)])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_xor[1] += 1
                    return
            # xor_minus_1: int_xor(-1, x) => int_invert(x)
            if C_arg_0 == -1:
                newop = self.replace_op_with(op, rop.INT_INVERT, args=[arg_1])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_xor[4] += 1
                return
            # xor_is_not: int_xor(1, x) => int_is_zero(x)
            if C_arg_0 == 1:
                if b_arg_1.is_bool():
                    newop = self.replace_op_with(op, rop.INT_IS_ZERO, args=[arg_1])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_xor[6] += 1
                    return
        else:
            arg_0_int_xor = self.optimizer.as_operation(arg_0, rop.INT_XOR)
            if arg_0_int_xor is not None:
                arg_0_0 = get_box_replacement(arg_0_int_xor.getarg(0))
                b_arg_0_0 = self.getintbound(arg_0_0)
                arg_0_1 = get_box_replacement(arg_0_int_xor.getarg(1))
                b_arg_0_1 = self.getintbound(arg_0_1)
                if b_arg_0_0.is_constant():
                    C_arg_0_0 = b_arg_0_0.get_constant_int()
                    if b_arg_1.is_constant():
                        C_arg_1 = b_arg_1.get_constant_int()
                        # xor_reassoc_consts: int_xor(int_xor(C1, x), C2) => int_xor(x, C)
                        C = C_arg_0_0 ^ C_arg_1
                        newop = self.replace_op_with(op, rop.INT_XOR, args=[arg_0_1, ConstInt(C)])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_xor[1] += 1
                        return
                if b_arg_0_1.is_constant():
                    C_arg_0_1 = b_arg_0_1.get_constant_int()
                    if b_arg_1.is_constant():
                        C_arg_1 = b_arg_1.get_constant_int()
                        # xor_reassoc_consts: int_xor(int_xor(x, C1), C2) => int_xor(x, C)
                        C = C_arg_0_1 ^ C_arg_1
                        newop = self.replace_op_with(op, rop.INT_XOR, args=[arg_0_0, ConstInt(C)])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_int_xor[1] += 1
                        return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # xor_minus_1: int_xor(x, -1) => int_invert(x)
            if C_arg_1 == -1:
                newop = self.replace_op_with(op, rop.INT_INVERT, args=[arg_0])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_xor[4] += 1
                return
            # xor_is_not: int_xor(x, 1) => int_is_zero(x)
            if C_arg_1 == 1:
                if b_arg_0.is_bool():
                    newop = self.replace_op_with(op, rop.INT_IS_ZERO, args=[arg_0])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_xor[6] += 1
                    return
        return self.emit(op)
    _rule_names_int_lshift = ['lshift_zero_x', 'lshift_x_zero', 'lshift_rshift_c_c', 'lshift_and_rshift', 'lshift_urshift_c_c', 'lshift_and_urshift', 'lshift_lshift_c_c']
    _rule_fired_int_lshift = [0] * 7
    _all_rules_fired.append(('int_lshift', _rule_names_int_lshift, _rule_fired_int_lshift))
    def optimize_INT_LSHIFT(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_1 = get_box_replacement(op.getarg(1))
        b_arg_1 = self.getintbound(arg_1)
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # lshift_zero_x: int_lshift(0, x) => 0
            if C_arg_0 == 0:
                self.make_constant_int(op, 0)
                self._rule_fired_int_lshift[0] += 1
                return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # lshift_x_zero: int_lshift(x, 0) => x
            if C_arg_1 == 0:
                self.make_equal_to(op, arg_0)
                self._rule_fired_int_lshift[1] += 1
                return
        arg_0_int_and = self.optimizer.as_operation(arg_0, rop.INT_AND)
        if arg_0_int_and is not None:
            arg_0_0 = get_box_replacement(arg_0_int_and.getarg(0))
            b_arg_0_0 = self.getintbound(arg_0_0)
            arg_0_1 = get_box_replacement(arg_0_int_and.getarg(1))
            b_arg_0_1 = self.getintbound(arg_0_1)
            if b_arg_0_0.is_constant():
                C_arg_0_0 = b_arg_0_0.get_constant_int()
                arg_0_1_int_rshift = self.optimizer.as_operation(arg_0_1, rop.INT_RSHIFT)
                if arg_0_1_int_rshift is not None:
                    arg_0_1_0 = get_box_replacement(arg_0_1_int_rshift.getarg(0))
                    b_arg_0_1_0 = self.getintbound(arg_0_1_0)
                    arg_0_1_1 = get_box_replacement(arg_0_1_int_rshift.getarg(1))
                    b_arg_0_1_1 = self.getintbound(arg_0_1_1)
                    if b_arg_0_1_1.is_constant():
                        C_arg_0_1_1 = b_arg_0_1_1.get_constant_int()
                        if b_arg_1.is_constant():
                            C_arg_1 = b_arg_1.get_constant_int()
                            # lshift_and_rshift: int_lshift(int_and(C2, int_rshift(x, C1)), C1) => int_and(x, C)
                            if C_arg_1 == C_arg_0_1_1:
                                if 0 <= C_arg_0_1_1 and C_arg_0_1_1 < LONG_BIT:
                                    C = C_arg_0_0 << C_arg_0_1_1
                                    newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_1_0, ConstInt(C)])
                                    self.optimizer.send_extra_operation(newop)
                                    self._rule_fired_int_lshift[3] += 1
                                    return
                else:
                    arg_0_1_uint_rshift = self.optimizer.as_operation(arg_0_1, rop.UINT_RSHIFT)
                    if arg_0_1_uint_rshift is not None:
                        arg_0_1_0 = get_box_replacement(arg_0_1_uint_rshift.getarg(0))
                        b_arg_0_1_0 = self.getintbound(arg_0_1_0)
                        arg_0_1_1 = get_box_replacement(arg_0_1_uint_rshift.getarg(1))
                        b_arg_0_1_1 = self.getintbound(arg_0_1_1)
                        if b_arg_0_1_1.is_constant():
                            C_arg_0_1_1 = b_arg_0_1_1.get_constant_int()
                            if b_arg_1.is_constant():
                                C_arg_1 = b_arg_1.get_constant_int()
                                # lshift_and_urshift: int_lshift(int_and(C2, uint_rshift(x, C1)), C1) => int_and(x, C)
                                if C_arg_1 == C_arg_0_1_1:
                                    if 0 <= C_arg_0_1_1 and C_arg_0_1_1 < LONG_BIT:
                                        C = C_arg_0_0 << C_arg_0_1_1
                                        newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_1_0, ConstInt(C)])
                                        self.optimizer.send_extra_operation(newop)
                                        self._rule_fired_int_lshift[5] += 1
                                        return
            else:
                arg_0_0_int_rshift = self.optimizer.as_operation(arg_0_0, rop.INT_RSHIFT)
                if arg_0_0_int_rshift is not None:
                    arg_0_0_0 = get_box_replacement(arg_0_0_int_rshift.getarg(0))
                    b_arg_0_0_0 = self.getintbound(arg_0_0_0)
                    arg_0_0_1 = get_box_replacement(arg_0_0_int_rshift.getarg(1))
                    b_arg_0_0_1 = self.getintbound(arg_0_0_1)
                    if b_arg_0_0_1.is_constant():
                        C_arg_0_0_1 = b_arg_0_0_1.get_constant_int()
                        if b_arg_0_1.is_constant():
                            C_arg_0_1 = b_arg_0_1.get_constant_int()
                            if b_arg_1.is_constant():
                                C_arg_1 = b_arg_1.get_constant_int()
                                # lshift_and_rshift: int_lshift(int_and(int_rshift(x, C1), C2), C1) => int_and(x, C)
                                if C_arg_1 == C_arg_0_0_1:
                                    if 0 <= C_arg_0_0_1 and C_arg_0_0_1 < LONG_BIT:
                                        C = C_arg_0_1 << C_arg_0_0_1
                                        newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_0_0, ConstInt(C)])
                                        self.optimizer.send_extra_operation(newop)
                                        self._rule_fired_int_lshift[3] += 1
                                        return
                else:
                    arg_0_0_uint_rshift = self.optimizer.as_operation(arg_0_0, rop.UINT_RSHIFT)
                    if arg_0_0_uint_rshift is not None:
                        arg_0_0_0 = get_box_replacement(arg_0_0_uint_rshift.getarg(0))
                        b_arg_0_0_0 = self.getintbound(arg_0_0_0)
                        arg_0_0_1 = get_box_replacement(arg_0_0_uint_rshift.getarg(1))
                        b_arg_0_0_1 = self.getintbound(arg_0_0_1)
                        if b_arg_0_0_1.is_constant():
                            C_arg_0_0_1 = b_arg_0_0_1.get_constant_int()
                            if b_arg_0_1.is_constant():
                                C_arg_0_1 = b_arg_0_1.get_constant_int()
                                if b_arg_1.is_constant():
                                    C_arg_1 = b_arg_1.get_constant_int()
                                    # lshift_and_urshift: int_lshift(int_and(uint_rshift(x, C1), C2), C1) => int_and(x, C)
                                    if C_arg_1 == C_arg_0_0_1:
                                        if 0 <= C_arg_0_0_1 and C_arg_0_0_1 < LONG_BIT:
                                            C = C_arg_0_1 << C_arg_0_0_1
                                            newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_0_0, ConstInt(C)])
                                            self.optimizer.send_extra_operation(newop)
                                            self._rule_fired_int_lshift[5] += 1
                                            return
        else:
            arg_0_int_lshift = self.optimizer.as_operation(arg_0, rop.INT_LSHIFT)
            if arg_0_int_lshift is not None:
                arg_0_0 = get_box_replacement(arg_0_int_lshift.getarg(0))
                b_arg_0_0 = self.getintbound(arg_0_0)
                arg_0_1 = get_box_replacement(arg_0_int_lshift.getarg(1))
                b_arg_0_1 = self.getintbound(arg_0_1)
                if b_arg_0_1.is_constant():
                    C_arg_0_1 = b_arg_0_1.get_constant_int()
                    if b_arg_1.is_constant():
                        C_arg_1 = b_arg_1.get_constant_int()
                        # lshift_lshift_c_c: int_lshift(int_lshift(x, C1), C2) => int_lshift(x, C)
                        if 0 <= C_arg_0_1 and C_arg_0_1 < LONG_BIT and 0 <= C_arg_1 and C_arg_1 < LONG_BIT:
                            C = intmask(r_uint(C_arg_0_1) + r_uint(C_arg_1))
                            if C < LONG_BIT:
                                newop = self.replace_op_with(op, rop.INT_LSHIFT, args=[arg_0_0, ConstInt(C)])
                                self.optimizer.send_extra_operation(newop)
                                self._rule_fired_int_lshift[6] += 1
                                return
            else:
                arg_0_int_rshift = self.optimizer.as_operation(arg_0, rop.INT_RSHIFT)
                if arg_0_int_rshift is not None:
                    arg_0_0 = get_box_replacement(arg_0_int_rshift.getarg(0))
                    b_arg_0_0 = self.getintbound(arg_0_0)
                    arg_0_1 = get_box_replacement(arg_0_int_rshift.getarg(1))
                    b_arg_0_1 = self.getintbound(arg_0_1)
                    if b_arg_0_1.is_constant():
                        C_arg_0_1 = b_arg_0_1.get_constant_int()
                        if b_arg_1.is_constant():
                            C_arg_1 = b_arg_1.get_constant_int()
                            # lshift_rshift_c_c: int_lshift(int_rshift(x, C1), C1) => int_and(x, C)
                            if C_arg_1 == C_arg_0_1:
                                if 0 <= C_arg_0_1 and C_arg_0_1 < LONG_BIT:
                                    C = -1 << C_arg_0_1
                                    newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_0, ConstInt(C)])
                                    self.optimizer.send_extra_operation(newop)
                                    self._rule_fired_int_lshift[2] += 1
                                    return
                else:
                    arg_0_uint_rshift = self.optimizer.as_operation(arg_0, rop.UINT_RSHIFT)
                    if arg_0_uint_rshift is not None:
                        arg_0_0 = get_box_replacement(arg_0_uint_rshift.getarg(0))
                        b_arg_0_0 = self.getintbound(arg_0_0)
                        arg_0_1 = get_box_replacement(arg_0_uint_rshift.getarg(1))
                        b_arg_0_1 = self.getintbound(arg_0_1)
                        if b_arg_0_1.is_constant():
                            C_arg_0_1 = b_arg_0_1.get_constant_int()
                            if b_arg_1.is_constant():
                                C_arg_1 = b_arg_1.get_constant_int()
                                # lshift_urshift_c_c: int_lshift(uint_rshift(x, C1), C1) => int_and(x, C)
                                if C_arg_1 == C_arg_0_1:
                                    if 0 <= C_arg_0_1 and C_arg_0_1 < LONG_BIT:
                                        C = -1 << C_arg_0_1
                                        newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_0, ConstInt(C)])
                                        self.optimizer.send_extra_operation(newop)
                                        self._rule_fired_int_lshift[4] += 1
                                        return
        return self.emit(op)
    _rule_names_int_rshift = ['rshift_zero_x', 'rshift_x_zero', 'rshift_known_result', 'rshift_lshift', 'rshift_rshift_c_c']
    _rule_fired_int_rshift = [0] * 5
    _all_rules_fired.append(('int_rshift', _rule_names_int_rshift, _rule_fired_int_rshift))
    def optimize_INT_RSHIFT(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_1 = get_box_replacement(op.getarg(1))
        b_arg_1 = self.getintbound(arg_1)
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # rshift_zero_x: int_rshift(0, x) => 0
            if C_arg_0 == 0:
                self.make_constant_int(op, 0)
                self._rule_fired_int_rshift[0] += 1
                return
        # rshift_known_result: int_rshift(a, b) => C
        bound = b_arg_0.rshift_bound(b_arg_1)
        if bound.is_constant():
            C = bound.get_constant_int()
            self.make_equal_to(op, ConstInt(C))
            self._rule_fired_int_rshift[2] += 1
            return
        arg_0_int_lshift = self.optimizer.as_operation(arg_0, rop.INT_LSHIFT)
        if arg_0_int_lshift is not None:
            arg_0_0 = get_box_replacement(arg_0_int_lshift.getarg(0))
            b_arg_0_0 = self.getintbound(arg_0_0)
            arg_0_1 = get_box_replacement(arg_0_int_lshift.getarg(1))
            b_arg_0_1 = self.getintbound(arg_0_1)
            # rshift_lshift: int_rshift(int_lshift(x, y), y) => x
            if self._eq(arg_1, b_arg_1, arg_0_1, b_arg_0_1):
                if b_arg_0_0.lshift_bound_cannot_overflow(b_arg_0_1):
                    self.make_equal_to(op, arg_0_0)
                    self._rule_fired_int_rshift[3] += 1
                    return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # rshift_x_zero: int_rshift(x, 0) => x
            if C_arg_1 == 0:
                self.make_equal_to(op, arg_0)
                self._rule_fired_int_rshift[1] += 1
                return
        arg_0_int_rshift = self.optimizer.as_operation(arg_0, rop.INT_RSHIFT)
        if arg_0_int_rshift is not None:
            arg_0_0 = get_box_replacement(arg_0_int_rshift.getarg(0))
            b_arg_0_0 = self.getintbound(arg_0_0)
            arg_0_1 = get_box_replacement(arg_0_int_rshift.getarg(1))
            b_arg_0_1 = self.getintbound(arg_0_1)
            if b_arg_0_1.is_constant():
                C_arg_0_1 = b_arg_0_1.get_constant_int()
                if b_arg_1.is_constant():
                    C_arg_1 = b_arg_1.get_constant_int()
                    # rshift_rshift_c_c: int_rshift(int_rshift(x, C1), C2) => int_rshift(x, C)
                    if 0 <= C_arg_0_1 and C_arg_0_1 < LONG_BIT and 0 <= C_arg_1 and C_arg_1 < LONG_BIT:
                        C = min(intmask(r_uint(C_arg_0_1) + r_uint(C_arg_1)), intmask(r_uint(LONG_BIT) - r_uint(1)))
                        if 0 <= C and C < LONG_BIT:
                            newop = self.replace_op_with(op, rop.INT_RSHIFT, args=[arg_0_0, ConstInt(C)])
                            self.optimizer.send_extra_operation(newop)
                            self._rule_fired_int_rshift[4] += 1
                            return
        return self.emit(op)
    _rule_names_uint_rshift = ['urshift_zero_x', 'urshift_x_zero', 'urshift_known_result', 'urshift_lshift_x_c_c']
    _rule_fired_uint_rshift = [0] * 4
    _all_rules_fired.append(('uint_rshift', _rule_names_uint_rshift, _rule_fired_uint_rshift))
    def optimize_UINT_RSHIFT(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_1 = get_box_replacement(op.getarg(1))
        b_arg_1 = self.getintbound(arg_1)
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # urshift_zero_x: uint_rshift(0, x) => 0
            if C_arg_0 == 0:
                self.make_constant_int(op, 0)
                self._rule_fired_uint_rshift[0] += 1
                return
        # urshift_known_result: uint_rshift(a, b) => C
        bound = b_arg_0.urshift_bound(b_arg_1)
        if bound.is_constant():
            C = bound.get_constant_int()
            self.make_equal_to(op, ConstInt(C))
            self._rule_fired_uint_rshift[2] += 1
            return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # urshift_x_zero: uint_rshift(x, 0) => x
            if C_arg_1 == 0:
                self.make_equal_to(op, arg_0)
                self._rule_fired_uint_rshift[1] += 1
                return
        arg_0_int_lshift = self.optimizer.as_operation(arg_0, rop.INT_LSHIFT)
        if arg_0_int_lshift is not None:
            arg_0_0 = get_box_replacement(arg_0_int_lshift.getarg(0))
            b_arg_0_0 = self.getintbound(arg_0_0)
            arg_0_1 = get_box_replacement(arg_0_int_lshift.getarg(1))
            b_arg_0_1 = self.getintbound(arg_0_1)
            if b_arg_0_1.is_constant():
                C_arg_0_1 = b_arg_0_1.get_constant_int()
                if b_arg_1.is_constant():
                    C_arg_1 = b_arg_1.get_constant_int()
                    # urshift_lshift_x_c_c: uint_rshift(int_lshift(x, C), C) => int_and(x, mask)
                    if C_arg_1 == C_arg_0_1:
                        mask = intmask(r_uint(-1 << C_arg_0_1) >> r_uint(C_arg_0_1))
                        newop = self.replace_op_with(op, rop.INT_AND, args=[arg_0_0, ConstInt(mask)])
                        self.optimizer.send_extra_operation(newop)
                        self._rule_fired_uint_rshift[3] += 1
                        return
        return self.emit(op)
    _rule_names_int_eq = ['eq_different_knownbits', 'eq_same', 'eq_one', 'eq_zero', 'eq_sub_eq']
    _rule_fired_int_eq = [0] * 5
    _all_rules_fired.append(('int_eq', _rule_names_int_eq, _rule_fired_int_eq))
    def optimize_INT_EQ(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_1 = get_box_replacement(op.getarg(1))
        b_arg_1 = self.getintbound(arg_1)
        arg_0_int_sub = self.optimizer.as_operation(arg_0, rop.INT_SUB)
        if arg_0_int_sub is not None:
            arg_0_0 = get_box_replacement(arg_0_int_sub.getarg(0))
            b_arg_0_0 = self.getintbound(arg_0_0)
            arg_0_1 = get_box_replacement(arg_0_int_sub.getarg(1))
            b_arg_0_1 = self.getintbound(arg_0_1)
            arg_0_1_int_eq = self.optimizer.as_operation(arg_0_1, rop.INT_EQ)
            if arg_0_1_int_eq is not None:
                arg_0_1_0 = get_box_replacement(arg_0_1_int_eq.getarg(0))
                b_arg_0_1_0 = self.getintbound(arg_0_1_0)
                arg_0_1_1 = get_box_replacement(arg_0_1_int_eq.getarg(1))
                b_arg_0_1_1 = self.getintbound(arg_0_1_1)
                # eq_sub_eq: int_eq(int_sub(x, int_eq(x, a)), a) => 0
                if self._eq(arg_0_1_0, b_arg_0_1_0, arg_0_0, b_arg_0_0):
                    if self._eq(arg_1, b_arg_1, arg_0_1_1, b_arg_0_1_1):
                        self.make_constant_int(op, 0)
                        self._rule_fired_int_eq[4] += 1
                        return
                # eq_sub_eq: int_eq(int_sub(x, int_eq(a, x)), a) => 0
                if self._eq(arg_0_1_1, b_arg_0_1_1, arg_0_0, b_arg_0_0):
                    if self._eq(arg_1, b_arg_1, arg_0_1_0, b_arg_0_1_0):
                        self.make_constant_int(op, 0)
                        self._rule_fired_int_eq[4] += 1
                        return
        arg_1_int_sub = self.optimizer.as_operation(arg_1, rop.INT_SUB)
        if arg_1_int_sub is not None:
            arg_1_0 = get_box_replacement(arg_1_int_sub.getarg(0))
            b_arg_1_0 = self.getintbound(arg_1_0)
            arg_1_1 = get_box_replacement(arg_1_int_sub.getarg(1))
            b_arg_1_1 = self.getintbound(arg_1_1)
            arg_1_1_int_eq = self.optimizer.as_operation(arg_1_1, rop.INT_EQ)
            if arg_1_1_int_eq is not None:
                arg_1_1_0 = get_box_replacement(arg_1_1_int_eq.getarg(0))
                b_arg_1_1_0 = self.getintbound(arg_1_1_0)
                arg_1_1_1 = get_box_replacement(arg_1_1_int_eq.getarg(1))
                b_arg_1_1_1 = self.getintbound(arg_1_1_1)
                # eq_sub_eq: int_eq(a, int_sub(x, int_eq(x, a))) => 0
                if self._eq(arg_1_1_0, b_arg_1_1_0, arg_1_0, b_arg_1_0):
                    if self._eq(arg_1_1_1, b_arg_1_1_1, arg_0, b_arg_0):
                        self.make_constant_int(op, 0)
                        self._rule_fired_int_eq[4] += 1
                        return
                # eq_sub_eq: int_eq(a, int_sub(x, int_eq(a, x))) => 0
                if self._eq(arg_1_1_0, b_arg_1_1_0, arg_0, b_arg_0):
                    if self._eq(arg_1_1_1, b_arg_1_1_1, arg_1_0, b_arg_1_0):
                        self.make_constant_int(op, 0)
                        self._rule_fired_int_eq[4] += 1
                        return
        # eq_same: int_eq(x, x) => 1
        if self._eq(arg_1, b_arg_1, arg_0, b_arg_0):
            self.make_constant_int(op, 1)
            self._rule_fired_int_eq[1] += 1
            return
        # eq_different_knownbits: int_eq(x, y) => 0
        if b_arg_0.known_ne(b_arg_1):
            self.make_constant_int(op, 0)
            self._rule_fired_int_eq[0] += 1
            return
        # eq_different_knownbits: int_eq(y, x) => 0
        if b_arg_1.known_ne(b_arg_0):
            self.make_constant_int(op, 0)
            self._rule_fired_int_eq[0] += 1
            return
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # eq_one: int_eq(1, x) => x
            if C_arg_0 == 1:
                if b_arg_1.is_bool():
                    self.make_equal_to(op, arg_1)
                    self._rule_fired_int_eq[2] += 1
                    return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # eq_one: int_eq(x, 1) => x
            if C_arg_1 == 1:
                if b_arg_0.is_bool():
                    self.make_equal_to(op, arg_0)
                    self._rule_fired_int_eq[2] += 1
                    return
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # eq_zero: int_eq(0, x) => int_is_zero(x)
            if C_arg_0 == 0:
                newop = self.replace_op_with(op, rop.INT_IS_ZERO, args=[arg_1])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_eq[3] += 1
                return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # eq_zero: int_eq(x, 0) => int_is_zero(x)
            if C_arg_1 == 0:
                newop = self.replace_op_with(op, rop.INT_IS_ZERO, args=[arg_0])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_eq[3] += 1
                return
        return self.emit(op)
    _rule_names_int_ne = ['ne_different_knownbits', 'ne_same', 'ne_zero']
    _rule_fired_int_ne = [0] * 3
    _all_rules_fired.append(('int_ne', _rule_names_int_ne, _rule_fired_int_ne))
    def optimize_INT_NE(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_1 = get_box_replacement(op.getarg(1))
        b_arg_1 = self.getintbound(arg_1)
        # ne_same: int_ne(x, x) => 0
        if self._eq(arg_1, b_arg_1, arg_0, b_arg_0):
            self.make_constant_int(op, 0)
            self._rule_fired_int_ne[1] += 1
            return
        # ne_different_knownbits: int_ne(x, y) => 1
        if b_arg_0.known_ne(b_arg_1):
            self.make_constant_int(op, 1)
            self._rule_fired_int_ne[0] += 1
            return
        # ne_different_knownbits: int_ne(y, x) => 1
        if b_arg_1.known_ne(b_arg_0):
            self.make_constant_int(op, 1)
            self._rule_fired_int_ne[0] += 1
            return
        if b_arg_0.is_constant():
            C_arg_0 = b_arg_0.get_constant_int()
            # ne_zero: int_ne(0, x) => int_is_true(x)
            if C_arg_0 == 0:
                newop = self.replace_op_with(op, rop.INT_IS_TRUE, args=[arg_1])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_ne[2] += 1
                return
        if b_arg_1.is_constant():
            C_arg_1 = b_arg_1.get_constant_int()
            # ne_zero: int_ne(x, 0) => int_is_true(x)
            if C_arg_1 == 0:
                newop = self.replace_op_with(op, rop.INT_IS_TRUE, args=[arg_0])
                self.optimizer.send_extra_operation(newop)
                self._rule_fired_int_ne[2] += 1
                return
        return self.emit(op)
    _rule_names_int_is_true = ['is_true_bool', 'is_true_true', 'is_true_and_minint']
    _rule_fired_int_is_true = [0] * 3
    _all_rules_fired.append(('int_is_true', _rule_names_int_is_true, _rule_fired_int_is_true))
    def optimize_INT_IS_TRUE(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        # is_true_true: int_is_true(x) => 1
        if intmask(b_arg_0.lower) > 0 or intmask(b_arg_0.upper) < 0 or intmask(b_arg_0.tvalue) != 0:
            self.make_constant_int(op, 1)
            self._rule_fired_int_is_true[1] += 1
            return
        # is_true_bool: int_is_true(x) => x
        if b_arg_0.is_bool():
            self.make_equal_to(op, arg_0)
            self._rule_fired_int_is_true[0] += 1
            return
        arg_0_int_and = self.optimizer.as_operation(arg_0, rop.INT_AND)
        if arg_0_int_and is not None:
            arg_0_0 = get_box_replacement(arg_0_int_and.getarg(0))
            b_arg_0_0 = self.getintbound(arg_0_0)
            arg_0_1 = get_box_replacement(arg_0_int_and.getarg(1))
            b_arg_0_1 = self.getintbound(arg_0_1)
            if b_arg_0_0.is_constant():
                C_arg_0_0 = b_arg_0_0.get_constant_int()
                # is_true_and_minint: int_is_true(int_and(MININT, x)) => int_lt(x, 0)
                if C_arg_0_0 == MININT:
                    newop = self.replace_op_with(op, rop.INT_LT, args=[arg_0_1, ConstInt(0)])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_is_true[2] += 1
                    return
            if b_arg_0_1.is_constant():
                C_arg_0_1 = b_arg_0_1.get_constant_int()
                # is_true_and_minint: int_is_true(int_and(x, MININT)) => int_lt(x, 0)
                if C_arg_0_1 == MININT:
                    newop = self.replace_op_with(op, rop.INT_LT, args=[arg_0_0, ConstInt(0)])
                    self.optimizer.send_extra_operation(newop)
                    self._rule_fired_int_is_true[2] += 1
                    return
        return self.emit(op)
    _rule_names_int_is_zero = ['is_zero_true']
    _rule_fired_int_is_zero = [0] * 1
    _all_rules_fired.append(('int_is_zero', _rule_names_int_is_zero, _rule_fired_int_is_zero))
    def optimize_INT_IS_ZERO(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        # is_zero_true: int_is_zero(x) => 0
        if intmask(b_arg_0.lower) > 0 or intmask(b_arg_0.upper) < 0 or intmask(b_arg_0.tvalue) != 0:
            self.make_constant_int(op, 0)
            self._rule_fired_int_is_zero[0] += 1
            return
        return self.emit(op)
    _rule_names_int_force_ge_zero = ['force_ge_zero_pos', 'force_ge_zero_neg']
    _rule_fired_int_force_ge_zero = [0] * 2
    _all_rules_fired.append(('int_force_ge_zero', _rule_names_int_force_ge_zero, _rule_fired_int_force_ge_zero))
    def optimize_INT_FORCE_GE_ZERO(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        # force_ge_zero_neg: int_force_ge_zero(x) => 0
        if b_arg_0.known_lt_const(0):
            self.make_constant_int(op, 0)
            self._rule_fired_int_force_ge_zero[1] += 1
            return
        # force_ge_zero_pos: int_force_ge_zero(x) => x
        if b_arg_0.known_nonnegative():
            self.make_equal_to(op, arg_0)
            self._rule_fired_int_force_ge_zero[0] += 1
            return
        return self.emit(op)
    _rule_names_int_invert = ['invert_invert']
    _rule_fired_int_invert = [0] * 1
    _all_rules_fired.append(('int_invert', _rule_names_int_invert, _rule_fired_int_invert))
    def optimize_INT_INVERT(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_0_int_invert = self.optimizer.as_operation(arg_0, rop.INT_INVERT)
        if arg_0_int_invert is not None:
            arg_0_0 = get_box_replacement(arg_0_int_invert.getarg(0))
            b_arg_0_0 = self.getintbound(arg_0_0)
            # invert_invert: int_invert(int_invert(x)) => x
            self.make_equal_to(op, arg_0_0)
            self._rule_fired_int_invert[0] += 1
            return
        return self.emit(op)
    _rule_names_int_neg = ['neg_neg']
    _rule_fired_int_neg = [0] * 1
    _all_rules_fired.append(('int_neg', _rule_names_int_neg, _rule_fired_int_neg))
    def optimize_INT_NEG(self, op):
        arg_0 = get_box_replacement(op.getarg(0))
        b_arg_0 = self.getintbound(arg_0)
        arg_0_int_neg = self.optimizer.as_operation(arg_0, rop.INT_NEG)
        if arg_0_int_neg is not None:
            arg_0_0 = get_box_replacement(arg_0_int_neg.getarg(0))
            b_arg_0_0 = self.getintbound(arg_0_0)
            # neg_neg: int_neg(int_neg(x)) => x
            self.make_equal_to(op, arg_0_0)
            self._rule_fired_int_neg[0] += 1
            return
        return self.emit(op)
